"""Relationshipモデルのテスト"""
from datetime import date
from uuid import uuid4
import pytest
from pydantic import ValidationError

from inheritance_calculator_core.models.relationship import (
    ChildOf,
    SpouseOf,
    SiblingOf,
    Renounced,
    Disqualified,
    Disinherited,
    BloodType,
    SharedParent,
)


class TestChildOf:
    """ChildOf関係のテスト"""

    def test_create_biological_child(self):
        """実子関係の作成"""
        parent_id = uuid4()
        child_id = uuid4()

        relationship = ChildOf(
            parent_id=parent_id,
            child_id=child_id,
            is_biological=True
        )

        assert relationship.parent_id == parent_id
        assert relationship.child_id == child_id
        assert relationship.is_biological is True
        assert relationship.is_adoption is False

    def test_create_adopted_child(self):
        """養子関係の作成"""
        parent_id = uuid4()
        child_id = uuid4()

        relationship = ChildOf(
            parent_id=parent_id,
            child_id=child_id,
            is_adoption=True,
            is_biological=False,
            adoption_date=date(2020, 4, 1)
        )

        assert relationship.is_adoption is True
        assert relationship.is_biological is False
        assert relationship.adoption_date == date(2020, 4, 1)

    def test_adopted_cannot_be_biological(self):
        """養子は実子ではないことの検証"""
        parent_id = uuid4()
        child_id = uuid4()

        with pytest.raises(ValidationError) as exc_info:
            ChildOf(
                parent_id=parent_id,
                child_id=child_id,
                is_adoption=True,
                is_biological=True
            )
        assert "Adopted child cannot be biological" in str(exc_info.value)


class TestSpouseOf:
    """SpouseOf関係のテスト"""

    def test_create_current_spouse(self):
        """現在の配偶者関係の作成"""
        person1_id = uuid4()
        person2_id = uuid4()

        relationship = SpouseOf(
            person1_id=person1_id,
            person2_id=person2_id,
            marriage_date=date(2010, 6, 1),
            is_current=True
        )

        assert relationship.person1_id == person1_id
        assert relationship.person2_id == person2_id
        assert relationship.marriage_date == date(2010, 6, 1)
        assert relationship.is_current is True
        assert relationship.divorce_date is None

    def test_create_divorced_spouse(self):
        """離婚した配偶者関係の作成"""
        person1_id = uuid4()
        person2_id = uuid4()

        relationship = SpouseOf(
            person1_id=person1_id,
            person2_id=person2_id,
            marriage_date=date(2010, 6, 1),
            divorce_date=date(2020, 12, 31),
            is_current=False
        )

        assert relationship.divorce_date == date(2020, 12, 31)
        assert relationship.is_current is False

    def test_divorced_cannot_be_current(self):
        """離婚日がある場合、is_currentはFalseでなければならない"""
        person1_id = uuid4()
        person2_id = uuid4()

        with pytest.raises(ValidationError) as exc_info:
            SpouseOf(
                person1_id=person1_id,
                person2_id=person2_id,
                divorce_date=date(2020, 12, 31),
                is_current=True
            )
        assert "Divorced spouse cannot be current" in str(exc_info.value)

    def test_divorce_date_before_marriage_invalid(self):
        """離婚日が婚姻日より前は不可"""
        person1_id = uuid4()
        person2_id = uuid4()

        with pytest.raises(ValidationError) as exc_info:
            SpouseOf(
                person1_id=person1_id,
                person2_id=person2_id,
                marriage_date=date(2010, 6, 1),
                divorce_date=date(2009, 12, 31),
                is_current=False
            )
        assert "Divorce date cannot be before marriage date" in str(exc_info.value)


class TestSiblingOf:
    """SiblingOf関係のテスト"""

    def test_create_full_blood_siblings(self):
        """全血兄弟姉妹関係の作成"""
        person1_id = uuid4()
        person2_id = uuid4()

        relationship = SiblingOf(
            person1_id=person1_id,
            person2_id=person2_id,
            blood_type=BloodType.FULL,
            shared_parent=SharedParent.BOTH
        )

        assert relationship.blood_type == BloodType.FULL
        assert relationship.shared_parent == SharedParent.BOTH

    def test_create_half_blood_siblings_father(self):
        """半血兄弟姉妹関係（父のみ同じ）の作成"""
        person1_id = uuid4()
        person2_id = uuid4()

        relationship = SiblingOf(
            person1_id=person1_id,
            person2_id=person2_id,
            blood_type=BloodType.HALF,
            shared_parent=SharedParent.FATHER
        )

        assert relationship.blood_type == BloodType.HALF
        assert relationship.shared_parent == SharedParent.FATHER

    def test_create_half_blood_siblings_mother(self):
        """半血兄弟姉妹関係（母のみ同じ）の作成"""
        person1_id = uuid4()
        person2_id = uuid4()

        relationship = SiblingOf(
            person1_id=person1_id,
            person2_id=person2_id,
            blood_type=BloodType.HALF,
            shared_parent=SharedParent.MOTHER
        )

        assert relationship.blood_type == BloodType.HALF
        assert relationship.shared_parent == SharedParent.MOTHER

    def test_full_blood_must_share_both_parents(self):
        """全血の場合、両親とも同じでなければならない"""
        person1_id = uuid4()
        person2_id = uuid4()

        with pytest.raises(ValidationError) as exc_info:
            SiblingOf(
                person1_id=person1_id,
                person2_id=person2_id,
                blood_type=BloodType.FULL,
                shared_parent=SharedParent.FATHER
            )
        assert "Full blood siblings must share both parents" in str(exc_info.value)

    def test_half_blood_cannot_share_both_parents(self):
        """半血の場合、両親とも同じではいけない"""
        person1_id = uuid4()
        person2_id = uuid4()

        with pytest.raises(ValidationError) as exc_info:
            SiblingOf(
                person1_id=person1_id,
                person2_id=person2_id,
                blood_type=BloodType.HALF,
                shared_parent=SharedParent.BOTH
            )
        assert "Half blood siblings cannot share both parents" in str(exc_info.value)


class TestRenounced:
    """Renounced（相続放棄）関係のテスト"""

    def test_create_renounced(self):
        """相続放棄関係の作成"""
        heir_id = uuid4()
        decedent_id = uuid4()

        relationship = Renounced(
            heir_id=heir_id,
            decedent_id=decedent_id,
            renounce_date=date(2025, 7, 1),
            reason="経済的理由"
        )

        assert relationship.heir_id == heir_id
        assert relationship.decedent_id == decedent_id
        assert relationship.renounce_date == date(2025, 7, 1)
        assert relationship.reason == "経済的理由"

    def test_renounce_date_future_invalid(self):
        """未来の放棄日は不可"""
        heir_id = uuid4()
        decedent_id = uuid4()
        from datetime import date as date_class, timedelta
        future_date = date_class.today() + timedelta(days=1)

        with pytest.raises(ValidationError) as exc_info:
            Renounced(
                heir_id=heir_id,
                decedent_id=decedent_id,
                renounce_date=future_date
            )
        assert "Renounce date cannot be in the future" in str(exc_info.value)


class TestDisqualified:
    """Disqualified（相続欠格）関係のテスト"""

    def test_create_disqualified(self):
        """相続欠格関係の作成"""
        heir_id = uuid4()
        decedent_id = uuid4()

        relationship = Disqualified(
            heir_id=heir_id,
            decedent_id=decedent_id,
            reason="被相続人を殺害した",
            determination_date=date(2025, 6, 1)
        )

        assert relationship.heir_id == heir_id
        assert relationship.decedent_id == decedent_id
        assert relationship.reason == "被相続人を殺害した"
        assert relationship.determination_date == date(2025, 6, 1)

    def test_reason_required(self):
        """欠格事由は必須"""
        heir_id = uuid4()
        decedent_id = uuid4()

        with pytest.raises(ValidationError):
            Disqualified(
                heir_id=heir_id,
                decedent_id=decedent_id,
                reason="",
                determination_date=date(2025, 6, 1)
            )

    def test_determination_date_future_invalid(self):
        """未来の確定日は不可"""
        heir_id = uuid4()
        decedent_id = uuid4()
        from datetime import date as date_class, timedelta
        future_date = date_class.today() + timedelta(days=1)

        with pytest.raises(ValidationError) as exc_info:
            Disqualified(
                heir_id=heir_id,
                decedent_id=decedent_id,
                reason="被相続人を殺害した",
                determination_date=future_date
            )
        assert "Determination date cannot be in the future" in str(exc_info.value)


class TestDisinherited:
    """Disinherited（相続廃除）関係のテスト"""

    def test_create_disinherited(self):
        """相続廃除関係の作成"""
        heir_id = uuid4()
        decedent_id = uuid4()

        relationship = Disinherited(
            heir_id=heir_id,
            decedent_id=decedent_id,
            reason="被相続人に対する虐待",
            court_decision_date=date(2025, 5, 1)
        )

        assert relationship.heir_id == heir_id
        assert relationship.decedent_id == decedent_id
        assert relationship.reason == "被相続人に対する虐待"
        assert relationship.court_decision_date == date(2025, 5, 1)

    def test_reason_required(self):
        """廃除事由は必須"""
        heir_id = uuid4()
        decedent_id = uuid4()

        with pytest.raises(ValidationError):
            Disinherited(
                heir_id=heir_id,
                decedent_id=decedent_id,
                reason="",
                court_decision_date=date(2025, 5, 1)
            )

    def test_court_decision_date_future_invalid(self):
        """未来の審判確定日は不可"""
        heir_id = uuid4()
        decedent_id = uuid4()
        from datetime import date as date_class, timedelta
        future_date = date_class.today() + timedelta(days=1)

        with pytest.raises(ValidationError) as exc_info:
            Disinherited(
                heir_id=heir_id,
                decedent_id=decedent_id,
                reason="被相続人に対する虐待",
                court_decision_date=future_date
            )
        assert "Court decision date cannot be in the future" in str(exc_info.value)



"""ServiceFactoryのテスト"""
import pytest

from inheritance_calculator_core.services.factory import ServiceFactory
from inheritance_calculator_core.services.heir_validator import HeirValidator
from inheritance_calculator_core.services.share_calculator import ShareCalculator
from inheritance_calculator_core.services.inheritance_calculator import InheritanceCalculator


class TestServiceFactory:
    """ServiceFactoryのテスト"""

    def test_get_heir_validator(self):
        """HeirValidatorインスタンスを取得できる"""
        factory = ServiceFactory()
        validator = factory.get_heir_validator()

        assert isinstance(validator, HeirValidator)

    def test_heir_validator_singleton(self):
        """HeirValidatorはシングルトンとして動作する"""
        factory = ServiceFactory()
        validator1 = factory.get_heir_validator()
        validator2 = factory.get_heir_validator()

        assert validator1 is validator2

    def test_get_share_calculator(self):
        """ShareCalculatorインスタンスを取得できる"""
        factory = ServiceFactory()
        calculator = factory.get_share_calculator()

        assert isinstance(calculator, ShareCalculator)

    def test_share_calculator_singleton(self):
        """ShareCalculatorはシングルトンとして動作する"""
        factory = ServiceFactory()
        calculator1 = factory.get_share_calculator()
        calculator2 = factory.get_share_calculator()

        assert calculator1 is calculator2

    def test_get_inheritance_calculator(self):
        """InheritanceCalculatorインスタンスを取得できる"""
        factory = ServiceFactory()
        calculator = factory.get_inheritance_calculator()

        assert isinstance(calculator, InheritanceCalculator)

    def test_inheritance_calculator_singleton(self):
        """InheritanceCalculatorはシングルトンとして動作する"""
        factory = ServiceFactory()
        calculator1 = factory.get_inheritance_calculator()
        calculator2 = factory.get_inheritance_calculator()

        assert calculator1 is calculator2

    def test_inheritance_calculator_dependency_injection(self):
        """InheritanceCalculatorに依存が正しく注入される"""
        factory = ServiceFactory()
        calculator = factory.get_inheritance_calculator()

        # 注入された依存がファクトリから取得したインスタンスと同一であることを確認
        assert calculator.validator is factory.get_heir_validator()
        assert calculator.calculator is factory.get_share_calculator()

    def test_reset(self):
        """reset()で全てのインスタンスがクリアされる"""
        factory = ServiceFactory()

        # インスタンスを取得
        validator1 = factory.get_heir_validator()
        calculator1 = factory.get_share_calculator()
        inheritance_calc1 = factory.get_inheritance_calculator()

        # リセット
        factory.reset()

        # 再度取得すると別のインスタンスになる
        validator2 = factory.get_heir_validator()
        calculator2 = factory.get_share_calculator()
        inheritance_calc2 = factory.get_inheritance_calculator()

        assert validator1 is not validator2
        assert calculator1 is not calculator2
        assert inheritance_calc1 is not inheritance_calc2

    def test_multiple_factories_independent(self):
        """複数のファクトリインスタンスは独立している"""
        factory1 = ServiceFactory()
        factory2 = ServiceFactory()

        calculator1 = factory1.get_inheritance_calculator()
        calculator2 = factory2.get_inheritance_calculator()

        # 異なるファクトリから取得したインスタンスは異なる
        assert calculator1 is not calculator2
        assert calculator1.validator is not calculator2.validator
        assert calculator1.calculator is not calculator2.calculator



"""Ollamaクライアント

Ollamaとの通信を管理し、LLMとのインタラクションを提供する。
"""
from typing import Optional, Dict, Any, List
import logging

import ollama
from ollama import ChatResponse

from ..utils.config import Settings
from ..utils.exceptions import ServiceException


class OllamaClient:
    """
    Ollamaクライアント

    gpt-oss:20bモデルとの通信を管理し、
    相続に関する質問応答機能を提供する。
    """

    def __init__(
        self,
        model: str = "gpt-oss:20b",
        host: Optional[str] = None,
        timeout: int = 120
    ) -> None:
        """
        初期化

        Args:
            model: 使用するモデル名
            host: OllamaホストURL（Noneの場合は設定から取得）
            timeout: タイムアウト時間（秒）
        """
        self.logger = logging.getLogger(__name__)

        # 設定の取得（Ollamaのみを初期化）
        if host is None:
            from ..utils.config import OllamaSettings
            ollama_settings = OllamaSettings()
            host = ollama_settings.host

        self.model = model
        self.host = host
        self.timeout = timeout

        # クライアントの初期化確認
        self._verify_connection()

        self.logger.info(
            f"OllamaClient initialized: model={self.model}, host={self.host}"
        )

    def _verify_connection(self) -> None:
        """
        Ollamaサーバーへの接続を確認

        Raises:
            ServiceException: 接続に失敗した場合
        """
        try:
            # モデルリストの取得で接続確認
            models = ollama.list()

            # モデルの存在確認
            model_names = [m.model for m in models.models if m.model is not None]
            if self.model not in model_names:
                available = ", ".join(model_names)
                raise ServiceException(
                    f"Model '{self.model}' not found. "
                    f"Available models: {available}"
                )

            self.logger.info(f"Connected to Ollama successfully")

        except Exception as e:
            error_msg = f"Failed to connect to Ollama: {str(e)}"
            self.logger.error(error_msg)
            raise ServiceException(error_msg) from e

    def chat(
        self,
        messages: List[Dict[str, str]],
        temperature: float = 0.7,
        max_tokens: Optional[int] = None
    ) -> str:
        """
        チャット形式でLLMと対話

        Args:
            messages: メッセージリスト（[{"role": "user", "content": "..."}]形式）
            temperature: 生成のランダム性（0.0-1.0）
            max_tokens: 最大トークン数

        Returns:
            LLMの応答テキスト

        Raises:
            ServiceException: 通信エラーが発生した場合
        """
        try:
            self.logger.debug(f"Sending chat request with {len(messages)} messages")

            # Ollamaのchat APIを呼び出し
            options: Dict[str, Any] = {
                "temperature": temperature,
            }
            if max_tokens:
                options["num_predict"] = max_tokens

            response: ChatResponse = ollama.chat(
                model=self.model,
                messages=messages,
                options=options
            )

            # 応答の取得
            content = response.message.content
            if content is None:
                self.logger.warning("Received None content from Ollama")
                return ""

            self.logger.debug(f"Received response: {len(content)} characters")

            return content

        except Exception as e:
            error_msg = f"Chat request failed: {str(e)}"
            self.logger.error(error_msg)
            raise ServiceException(error_msg) from e

    def generate(
        self,
        prompt: str,
        system: Optional[str] = None,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None
    ) -> str:
        """
        プロンプトから文章を生成

        Args:
            prompt: プロンプトテキスト
            system: システムプロンプト（オプション）
            temperature: 生成のランダム性（0.0-1.0）
            max_tokens: 最大トークン数

        Returns:
            生成されたテキスト

        Raises:
            ServiceException: 通信エラーが発生した場合
        """
        messages: List[Dict[str, str]] = []

        if system:
            messages.append({"role": "system", "content": system})

        messages.append({"role": "user", "content": prompt})

        return self.chat(
            messages=messages,
            temperature=temperature,
            max_tokens=max_tokens
        )

    def ask_question(
        self,
        question: str,
        context: Optional[str] = None
    ) -> str:
        """
        質問に対する回答を生成

        Args:
            question: 質問テキスト
            context: 質問のコンテキスト（会話履歴など）

        Returns:
            回答テキスト

        Raises:
            ServiceException: 通信エラーが発生した場合
        """
        system_prompt = """あなたは日本の相続法に詳しい専門家です。
被相続人の相続に関する情報を収集するために、ユーザーに質問をします。
回答は簡潔で分かりやすく、必要に応じて法的な説明を加えてください。"""

        if context:
            prompt = f"コンテキスト: {context}\n\n質問: {question}"
        else:
            prompt = question

        return self.generate(
            prompt=prompt,
            system=system_prompt,
            temperature=0.3  # 法的情報なので低めの温度
        )

    def parse_user_input(
        self,
        user_input: str,
        expected_format: str
    ) -> str:
        """
        ユーザー入力を解析して構造化

        Args:
            user_input: ユーザーの入力
            expected_format: 期待される形式の説明

        Returns:
            解析結果（JSON形式の文字列など）

        Raises:
            ServiceException: 解析エラーが発生した場合
        """
        system_prompt = f"""ユーザーの入力を解析して、以下の形式で出力してください。
出力形式: {expected_format}

余計な説明は不要です。指定された形式のみを出力してください。"""

        return self.generate(
            prompt=user_input,
            system=system_prompt,
            temperature=0.1  # 解析タスクなので低温度
        )



"""関係性モデル"""
from datetime import date
from typing import Optional, Any
from enum import Enum
from uuid import UUID

from pydantic import Field, field_validator

from .base import Neo4jRelationship


class BloodType(str, Enum):
    """血縁タイプ"""
    FULL = "full"  # 全血（父母が同じ）
    HALF = "half"  # 半血（父または母が同じ）


class SharedParent(str, Enum):
    """共通の親"""
    BOTH = "both"  # 両親とも同じ
    MOTHER = "mother"  # 母のみ同じ
    FATHER = "father"  # 父のみ同じ


class ChildOf(Neo4jRelationship):
    """
    子関係（CHILD_OF）

    子から親への関係を表現。
    """

    parent_id: UUID = Field(..., description="親のID")
    child_id: UUID = Field(..., description="子のID")
    is_adoption: bool = Field(default=False, description="養子縁組であるか")
    is_biological: bool = Field(default=True, description="実子であるか")
    adoption_date: Optional[date] = Field(default=None, description="養子縁組日")

    @field_validator('is_biological')
    @classmethod
    def validate_biological(cls, v: bool, info: Any) -> bool:
        """実子フラグの検証"""
        # 養子の場合、実子ではない
        values = info.data
        if values.get('is_adoption', False) and v:
            raise ValueError("Adopted child cannot be biological")
        return v


class SpouseOf(Neo4jRelationship):
    """
    配偶者関係（SPOUSE_OF）

    配偶者間の関係を表現。
    """

    person1_id: UUID = Field(..., description="配偶者1のID")
    person2_id: UUID = Field(..., description="配偶者2のID")
    marriage_date: Optional[date] = Field(default=None, description="婚姻日")
    divorce_date: Optional[date] = Field(default=None, description="離婚日")
    is_current: bool = Field(default=True, description="現在の配偶者か")

    @field_validator('is_current')
    @classmethod
    def validate_current(cls, v: bool, info: Any) -> bool:
        """現在の配偶者フラグの検証"""
        values = info.data
        # 離婚日がある場合、is_currentはFalseであるべき
        if values.get('divorce_date') is not None and v:
            raise ValueError("Divorced spouse cannot be current")
        return v

    @field_validator('divorce_date')
    @classmethod
    def validate_divorce_date(cls, v: Optional[date], info: Any) -> Optional[date]:
        """離婚日の検証"""
        if v is not None:
            values = info.data
            marriage_date = values.get('marriage_date')
            if marriage_date is not None and v < marriage_date:
                raise ValueError("Divorce date cannot be before marriage date")
        return v


class SiblingOf(Neo4jRelationship):
    """
    兄弟姉妹関係（SIBLING_OF）

    兄弟姉妹間の関係を表現。
    """

    person1_id: UUID = Field(..., description="人物1のID")
    person2_id: UUID = Field(..., description="人物2のID")
    blood_type: BloodType = Field(..., description="血縁タイプ")
    shared_parent: SharedParent = Field(..., description="共通の親")

    @field_validator('shared_parent')
    @classmethod
    def validate_shared_parent(cls, v: SharedParent, info: Any) -> SharedParent:
        """共通の親の検証"""
        values = info.data
        blood_type = values.get('blood_type')

        # 全血の場合、両親とも同じでなければならない
        if blood_type == BloodType.FULL and v != SharedParent.BOTH:
            raise ValueError("Full blood siblings must share both parents")

        # 半血の場合、片親のみ同じでなければならない
        if blood_type == BloodType.HALF and v == SharedParent.BOTH:
            raise ValueError("Half blood siblings cannot share both parents")

        return v


class Renounced(Neo4jRelationship):
    """
    相続放棄（RENOUNCED）

    相続放棄の関係を表現。
    """

    heir_id: UUID = Field(..., description="相続人のID")
    decedent_id: UUID = Field(..., description="被相続人のID")
    renounce_date: date = Field(..., description="放棄日")
    reason: Optional[str] = Field(default=None, description="放棄理由")

    @field_validator('renounce_date')
    @classmethod
    def validate_renounce_date(cls, v: date) -> date:
        """放棄日の検証"""
        from datetime import date as date_class
        # 未来の日付は不可
        if v > date_class.today():
            raise ValueError("Renounce date cannot be in the future")
        return v


class Disqualified(Neo4jRelationship):
    """
    相続欠格（DISQUALIFIED）

    相続欠格の関係を表現。
    """

    heir_id: UUID = Field(..., description="相続人のID")
    decedent_id: UUID = Field(..., description="被相続人のID")
    reason: str = Field(..., description="欠格事由", min_length=1)
    determination_date: date = Field(..., description="確定日")

    @field_validator('determination_date')
    @classmethod
    def validate_determination_date(cls, v: date) -> date:
        """確定日の検証"""
        from datetime import date as date_class
        if v > date_class.today():
            raise ValueError("Determination date cannot be in the future")
        return v


class Disinherited(Neo4jRelationship):
    """
    相続廃除（DISINHERITED）

    相続廃除の関係を表現。
    """

    heir_id: UUID = Field(..., description="相続人のID")
    decedent_id: UUID = Field(..., description="被相続人のID")
    reason: str = Field(..., description="廃除事由", min_length=1)
    court_decision_date: date = Field(..., description="審判確定日")

    @field_validator('court_decision_date')
    @classmethod
    def validate_court_decision_date(cls, v: date) -> date:
        """審判確定日の検証"""
        from datetime import date as date_class
        if v > date_class.today():
            raise ValueError("Court decision date cannot be in the future")
        return v



"""相続人資格検証サービス

日本の民法に基づいて相続人の資格を判定するサービス。
"""
from typing import List, Optional
from datetime import date

from ..models.person import Person
from ..models.relationship import (
    ChildOf,
    SpouseOf,
    SiblingOf,
    Renounced,
    Disqualified,
    Disinherited,
    BloodType,
)
from ..models.inheritance import HeritageRank, SubstitutionType
from .base import BaseService


class HeirValidator(BaseService[Person]):
    """
    相続人資格検証サービス

    相続人の資格を判定し、相続順位を確定する。
    """

    def __init__(self) -> None:
        """初期化"""
        super().__init__()
        self.decedent: Optional[Person] = None
        self.spouses: List[Person] = []
        self.children: List[Person] = []
        self.parents: List[Person] = []
        self.siblings: List[Person] = []
        self.renounced_persons: List[Person] = []
        self.disqualified_persons: List[Person] = []
        self.disinherited_persons: List[Person] = []

    def set_decedent(self, decedent: Person) -> None:
        """
        被相続人を設定

        Args:
            decedent: 被相続人
        """
        if not decedent.is_decedent:
            self.log_warning("Person is not marked as decedent", person=decedent.name)

        self.decedent = decedent
        self.log_operation("Set decedent", name=decedent.name)

    def is_valid_heir(self, person: Person) -> bool:
        """
        相続人として有効かチェック

        Args:
            person: チェック対象の人物

        Returns:
            相続人として有効な場合True
        """
        if self.decedent is None:
            raise ValueError("Decedent must be set before validating heirs")

        # 被相続人本人は相続人になれない
        if person.id == self.decedent.id:
            return False

        # 死亡している場合の判定
        if not person.is_alive:
            # 遺産分割前に死亡した場合は再転相続の対象として一旦有効とする
            # （後で再転相続処理で実際の相続人に置き換える）
            if person.died_before_division:
                return True
            # それ以外の死亡者は相続人になれない（代襲相続を除く）
            return False

        # 相続放棄している場合は相続人になれない
        if person in self.renounced_persons:
            self.log_info("Person has renounced inheritance", person=person.name)
            return False

        # 相続欠格の場合は相続人になれない（ただし代襲相続は可能）
        if person in self.disqualified_persons:
            self.log_info("Person is disqualified", person=person.name)
            return False

        # 相続廃除の場合は相続人になれない（ただし代襲相続は可能）
        if person in self.disinherited_persons:
            self.log_info("Person is disinherited", person=person.name)
            return False

        return True

    def validate_spouse(self, spouse: Person) -> bool:
        """
        配偶者の相続資格を検証

        民法890条：配偶者は常に相続人となる

        Args:
            spouse: 配偶者

        Returns:
            相続人として有効な場合True
        """
        if not self.is_valid_heir(spouse):
            return False

        # 配偶者は常に相続人（民法890条）
        self.log_operation(
            "Validated spouse",
            spouse=spouse.name,
            basis="民法890条"
        )
        return True

    def validate_child(self, child: Person) -> bool:
        """
        子の相続資格を検証

        民法887条1項：子は第1順位の相続人

        Args:
            child: 子

        Returns:
            相続人として有効な場合True
        """
        if not self.is_valid_heir(child):
            return False

        # 子は第1順位の相続人（民法887条1項）
        self.log_operation(
            "Validated child",
            child=child.name,
            basis="民法887条1項"
        )
        return True

    def validate_parent(self, parent: Person, has_first_rank: bool) -> bool:
        """
        直系尊属の相続資格を検証

        民法889条1項1号：第1順位の相続人がいない場合のみ相続

        Args:
            parent: 直系尊属（父母、祖父母等）
            has_first_rank: 第1順位の相続人が存在するか

        Returns:
            相続人として有効な場合True
        """
        if not self.is_valid_heir(parent):
            return False

        # 第1順位の相続人がいる場合は相続できない
        if has_first_rank:
            self.log_info(
                "Parent cannot inherit (first rank heirs exist)",
                parent=parent.name
            )
            return False

        # 直系尊属は第2順位の相続人（民法889条1項1号）
        self.log_operation(
            "Validated parent",
            parent=parent.name,
            basis="民法889条1項1号"
        )
        return True

    def validate_sibling(
        self,
        sibling: Person,
        has_first_rank: bool,
        has_second_rank: bool
    ) -> bool:
        """
        兄弟姉妹の相続資格を検証

        民法889条1項2号：第1順位、第2順位の相続人がいない場合のみ相続

        Args:
            sibling: 兄弟姉妹
            has_first_rank: 第1順位の相続人が存在するか
            has_second_rank: 第2順位の相続人が存在するか

        Returns:
            相続人として有効な場合True
        """
        if not self.is_valid_heir(sibling):
            return False

        # 第1順位または第2順位の相続人がいる場合は相続できない
        if has_first_rank or has_second_rank:
            self.log_info(
                "Sibling cannot inherit (higher rank heirs exist)",
                sibling=sibling.name
            )
            return False

        # 兄弟姉妹は第3順位の相続人（民法889条1項2号）
        self.log_operation(
            "Validated sibling",
            sibling=sibling.name,
            basis="民法889条1項2号"
        )
        return True

    def can_substitute(
        self,
        deceased_heir: Person,
        substitute: Person,
        rank: HeritageRank
    ) -> bool:
        """
        代襲相続が可能かチェック

        民法887条2項・3項：子の代襲相続（制限なし）
        民法889条2項：兄弟姉妹の代襲相続（1代限り）

        Args:
            deceased_heir: 死亡した相続人（被代襲者）
            substitute: 代襲相続人候補
            rank: 相続順位

        Returns:
            代襲相続が可能な場合True
        """
        if self.decedent is None:
            raise ValueError("Decedent must be set")

        # 代襲相続人候補が生存していなければならない
        if not substitute.is_alive:
            return False

        # 被代襲者が被相続人より先に死亡しているか、
        # 相続欠格・相続廃除されている必要がある
        valid_substitution_reason = (
            (deceased_heir.death_date is not None and
             self.decedent.death_date is not None and
             deceased_heir.death_date < self.decedent.death_date) or
            deceased_heir in self.disqualified_persons or
            deceased_heir in self.disinherited_persons
        )

        if not valid_substitution_reason:
            return False

        # 相続放棄は代襲原因にならない
        if deceased_heir in self.renounced_persons:
            self.log_info(
                "No substitution for renounced heir",
                deceased=deceased_heir.name
            )
            return False

        # 第1順位（子）の代襲：制限なし（民法887条2項・3項）
        if rank == HeritageRank.FIRST:
            self.log_operation(
                "Substitution allowed for child",
                substitute=substitute.name,
                deceased=deceased_heir.name,
                basis="民法887条2項・3項"
            )
            return True

        # 第3順位（兄弟姉妹）の代襲：1代限り（民法889条2項）
        if rank == HeritageRank.THIRD:
            # 兄弟姉妹の子（甥・姪）までのみ代襲可能
            self.log_operation(
                "Substitution allowed for sibling (1 generation only)",
                substitute=substitute.name,
                deceased=deceased_heir.name,
                basis="民法889条2項"
            )
            return True

        return False

    def get_substitution_type(self, rank: HeritageRank) -> SubstitutionType:
        """
        代襲相続のタイプを取得

        Args:
            rank: 相続順位

        Returns:
            代襲相続タイプ
        """
        if rank == HeritageRank.FIRST:
            return SubstitutionType.CHILD
        elif rank == HeritageRank.THIRD:
            return SubstitutionType.SIBLING
        else:
            return SubstitutionType.NONE



"""再転相続における相続放棄制約のテスト"""
from datetime import date
import pytest

from inheritance_calculator_core.models.person import Person
from inheritance_calculator_core.services.inheritance_calculator import InheritanceCalculator
from inheritance_calculator_core.utils.exceptions import RenunciationConflictError


class TestRetransferRenunciationConstraint:
    """再転相続における相続放棄制約のテストクラス（最高裁昭和63年6月21日判決）"""

    def test_second_inheritance_renounced_cannot_accept_first_only(self) -> None:
        """第2次相続を放棄した者は第1次相続のみを承認できない（判例違反ケース）

        ケース: 再転相続放棄制約
        - 被相続人A（2025年1月死亡）
        - 子B（2025年2月死亡、遺産分割前）
        - Bの子C（存命）
        - CがBの相続を放棄したが、Aの相続のみを承認しようとする
        - 期待結果: RenunciationConflictError が発生
        """
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C
        grandchild_c = Person(
            name="C",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # 再転相続先の情報: CはAの相続を承認しようとしている
        retransfer_info = {
            child_b.id: [grandchild_c]
        }

        # 再転相続先の関係情報
        retransfer_relationships = {
            child_b.id: {
                grandchild_c.id: 'child'
            }
        }

        # 第2次相続（Bの相続）をCが放棄
        second_inheritance_renounced = {
            child_b.id: [grandchild_c]
        }

        # 検証: 判例制約違反のためRenunciationConflictErrorが発生すべき
        with pytest.raises(RenunciationConflictError) as exc_info:
            calculator.calculate(
                decedent=decedent,
                spouses=[],
                children=[child_b],
                parents=[],
                siblings=[],
                retransfer_heirs_info=retransfer_info,
                retransfer_relationships=retransfer_relationships,
                second_inheritance_renounced=second_inheritance_renounced
            )

        # エラーメッセージの検証
        error_message = str(exc_info.value)
        assert "Cは" in error_message
        assert "Bの相続を放棄している" in error_message
        assert "Aの相続のみを承認することはできません" in error_message
        assert "最高裁昭和63年6月21日判決" in error_message

    def test_both_inheritances_renounced_is_allowed(self) -> None:
        """両方の相続を放棄することは許可される"""
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C（両方放棄）
        grandchild_c = Person(
            name="C",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # CはAの相続もBの相続も放棄する場合、再転相続先には含まれない
        retransfer_info = {
            child_b.id: []  # Cは含まれていない
        }

        # 第2次相続（Bの相続）をCが放棄
        second_inheritance_renounced = {
            child_b.id: [grandchild_c]
        }

        # 検証: エラーが発生しない（両方放棄は許可される）
        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child_b],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info,
            second_inheritance_renounced=second_inheritance_renounced
        )

        # 再転相続先がいない場合、Bが相続人として残る（died_before_divisionだが再転相続先なし）
        assert result.total_heirs == 1
        assert result.heirs[0].person.name == "B"

    def test_both_inheritances_accepted_is_allowed(self) -> None:
        """両方の相続を承認することは許可される"""
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C（両方承認）
        grandchild_c = Person(
            name="C",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # CはAの相続もBの相続も承認
        retransfer_info = {
            child_b.id: [grandchild_c]
        }

        retransfer_relationships = {
            child_b.id: {
                grandchild_c.id: 'child'
            }
        }

        # 第2次相続（Bの相続）の放棄者なし
        second_inheritance_renounced = {
            child_b.id: []
        }

        # 検証: エラーが発生しない（両方承認は許可される）
        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child_b],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info,
            retransfer_relationships=retransfer_relationships,
            second_inheritance_renounced=second_inheritance_renounced
        )

        # Cが全部相続
        assert result.total_heirs == 1
        assert result.heirs[0].person.name == "C"

    def test_first_inheritance_renounced_is_allowed(self) -> None:
        """第1次相続を放棄し、第2次相続を承認することは許可される"""
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C
        grandchild_c = Person(
            name="C",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # CがAの相続を放棄した場合、再転相続先には含まれない
        # （Aの相続権がないので、Bから承継する権利もない）
        retransfer_info = {
            child_b.id: []  # Cは含まれていない（Aの相続を放棄）
        }

        # Bの相続は承認（放棄者なし）
        second_inheritance_renounced = {
            child_b.id: []
        }

        # 検証: エラーが発生しない
        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child_b],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info,
            second_inheritance_renounced=second_inheritance_renounced
        )

        # 再転相続先がいない場合、Bが相続人として残る
        assert result.total_heirs == 1
        assert result.heirs[0].person.name == "B"

    def test_multiple_retransfer_heirs_partial_renunciation(self) -> None:
        """複数の再転相続先のうち一部が第2次相続を放棄したケース"""
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C（Bの相続を放棄）
        grandchild_c = Person(
            name="C",
            is_alive=True
        )

        # Bの子D（Bの相続を承認）
        grandchild_d = Person(
            name="D",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # CとDの両方がAの相続を承認しようとする（ただしCはBの相続を放棄）
        retransfer_info = {
            child_b.id: [grandchild_c, grandchild_d]
        }

        retransfer_relationships = {
            child_b.id: {
                grandchild_c.id: 'child',
                grandchild_d.id: 'child'
            }
        }

        # CがBの相続を放棄
        second_inheritance_renounced = {
            child_b.id: [grandchild_c]
        }

        # 検証: Cが含まれているためRenunciationConflictErrorが発生すべき
        with pytest.raises(RenunciationConflictError) as exc_info:
            calculator.calculate(
                decedent=decedent,
                spouses=[],
                children=[child_b],
                parents=[],
                siblings=[],
                retransfer_heirs_info=retransfer_info,
                retransfer_relationships=retransfer_relationships,
                second_inheritance_renounced=second_inheritance_renounced
            )

        error_message = str(exc_info.value)
        assert "C" in error_message



"""Neo4jクライアント

Neo4jデータベースへの接続と操作を管理するクライアント。
"""
from typing import Any, Dict, List, Optional, Generator
from contextlib import contextmanager
import logging

from neo4j import GraphDatabase, Driver, Session, Transaction
from neo4j.exceptions import ServiceUnavailable, AuthError, Neo4jError

from .base import DatabaseClient
from ..utils.exceptions import DatabaseException
from ..utils.config import Neo4jSettings


class Neo4jClient(DatabaseClient):
    """
    Neo4jデータベースクライアント

    接続管理、トランザクション管理、クエリ実行を提供する。
    """

    def __init__(self, settings: Optional[Neo4jSettings] = None) -> None:
        """
        初期化

        Args:
            settings: Neo4j接続設定（Noneの場合は環境変数から取得）
        """
        self.logger = logging.getLogger(__name__)

        if settings is None:
            try:
                settings = Neo4jSettings()  # type: ignore[call-arg]
            except Exception:
                # テスト環境等でパスワードが設定されていない場合のフォールバック
                raise DatabaseException("Neo4j設定の初期化に失敗しました。NEO4J_PASSWORD環境変数が設定されているか確認してください。")

        self.uri = settings.uri
        self.user = settings.user
        self.password = settings.password
        self.database = settings.database

        self._driver: Optional[Driver] = None
        self._session: Optional[Session] = None
        self._transaction: Optional[Transaction] = None

        self.logger.info(f"Neo4jClient initialized for {self.uri}")

    def connect(self) -> None:
        """
        データベースに接続

        Raises:
            DatabaseException: 接続に失敗した場合
        """
        try:
            self._driver = GraphDatabase.driver(
                self.uri,
                auth=(self.user, self.password),
                encrypted=False  # ローカル環境のため暗号化なし
            )

            # 接続テスト
            self._driver.verify_connectivity()

            self.logger.info("Successfully connected to Neo4j")

        except AuthError as e:
            self.logger.error(f"Authentication failed: {e}")
            raise DatabaseException(f"Neo4j認証に失敗しました: {str(e)}")
        except ServiceUnavailable as e:
            self.logger.error(f"Neo4j service unavailable: {e}")
            raise DatabaseException(f"Neo4jサービスに接続できません: {str(e)}")
        except Exception as e:
            self.logger.error(f"Failed to connect to Neo4j: {e}", exc_info=True)
            raise DatabaseException(f"Neo4j接続エラー: {str(e)}")

    def disconnect(self) -> None:
        """
        データベースから切断
        """
        try:
            # トランザクションをロールバック
            if self._transaction is not None:
                self._transaction.rollback()
                self._transaction = None

            # セッションを閉じる
            if self._session is not None:
                self._session.close()
                self._session = None

            # ドライバーを閉じる
            if self._driver is not None:
                self._driver.close()
                self._driver = None

            self.logger.info("Disconnected from Neo4j")

        except Exception as e:
            self.logger.error(f"Error during disconnect: {e}", exc_info=True)

    def is_connected(self) -> bool:
        """
        接続状態を確認

        Returns:
            接続中の場合True
        """
        if self._driver is None:
            return False

        try:
            self._driver.verify_connectivity()
            return True
        except Exception:
            return False

    def health_check(self) -> bool:
        """
        ヘルスチェック

        Returns:
            データベースが正常に動作している場合True
        """
        if self._driver is None:
            return False

        try:
            with self._driver.session(database=self.database) as session:
                result = session.run("RETURN 1 as health")
                record = result.single()
                if record is None:
                    return False
                return bool(record["health"] == 1)
        except Exception as e:
            self.logger.error(f"Health check failed: {e}")
            return False

    def begin_transaction(self) -> Transaction:
        """
        トランザクションを開始

        Returns:
            トランザクションオブジェクト

        Raises:
            DatabaseException: トランザクション開始に失敗した場合
        """
        if self._driver is None:
            raise DatabaseException("データベースに接続されていません")

        try:
            if self._session is None:
                self._session = self._driver.session(database=self.database)

            self._transaction = self._session.begin_transaction()
            self.logger.debug("Transaction started")
            return self._transaction

        except Exception as e:
            self.logger.error(f"Failed to begin transaction: {e}", exc_info=True)
            raise DatabaseException(f"トランザクション開始エラー: {str(e)}")

    def commit_transaction(self, transaction: Any) -> None:
        """
        トランザクションをコミット

        Args:
            transaction: トランザクションオブジェクト

        Raises:
            DatabaseException: コミットに失敗した場合
        """
        if transaction is None:
            raise DatabaseException("トランザクションがNullです")

        try:
            transaction.commit()
            if self._transaction == transaction:
                self._transaction = None
            self.logger.debug("Transaction committed")

        except Exception as e:
            self.logger.error(f"Failed to commit transaction: {e}", exc_info=True)
            raise DatabaseException(f"トランザクションコミットエラー: {str(e)}")

    def rollback_transaction(self, transaction: Any) -> None:
        """
        トランザクションをロールバック

        Args:
            transaction: トランザクションオブジェクト

        Raises:
            DatabaseException: ロールバックに失敗した場合
        """
        if transaction is None:
            raise DatabaseException("トランザクションがNullです")

        try:
            transaction.rollback()
            if self._transaction == transaction:
                self._transaction = None
            self.logger.debug("Transaction rolled back")

        except Exception as e:
            self.logger.error(f"Failed to rollback transaction: {e}", exc_info=True)
            raise DatabaseException(f"トランザクションロールバックエラー: {str(e)}")

    def commit(self) -> None:
        """
        現在のトランザクションをコミット（内部メソッド）

        Raises:
            DatabaseException: コミットに失敗した場合
        """
        if self._transaction is None:
            raise DatabaseException("アクティブなトランザクションがありません")
        self.commit_transaction(self._transaction)

    def rollback(self) -> None:
        """
        現在のトランザクションをロールバック（内部メソッド）

        Raises:
            DatabaseException: ロールバックに失敗した場合
        """
        if self._transaction is None:
            raise DatabaseException("アクティブなトランザクションがありません")
        self.rollback_transaction(self._transaction)

    def execute_query(
        self,
        query: str,
        parameters: Optional[Dict[str, Any]] = None
    ) -> List[Dict[str, Any]]:
        """
        Cypherクエリを実行（基底クラスメソッド）

        Args:
            query: Cypherクエリ文字列
            parameters: クエリパラメータ

        Returns:
            クエリ結果のリスト

        Raises:
            DatabaseException: クエリ実行に失敗した場合
        """
        if self._driver is None:
            raise DatabaseException("データベースに接続されていません")

        try:
            # トランザクション内の場合
            if self._transaction is not None:
                result = self._transaction.run(query, parameters or {})
                return [dict(record) for record in result]

            # トランザクション外の場合
            with self._driver.session(database=self.database) as session:
                result = session.run(query, parameters or {})
                return [dict(record) for record in result]

        except Neo4jError as e:
            self.logger.error(f"Neo4j query error: {e}", exc_info=True)
            raise DatabaseException(f"クエリ実行エラー: {str(e)}")
        except Exception as e:
            self.logger.error(f"Unexpected error during query execution: {e}", exc_info=True)
            raise DatabaseException(f"予期しないエラー: {str(e)}")

    def execute(
        self,
        query: str,
        parameters: Optional[Dict[str, Any]] = None
    ) -> List[Dict[str, Any]]:
        """
        Cypherクエリを実行（互換性メソッド）

        Args:
            query: Cypherクエリ文字列
            parameters: クエリパラメータ

        Returns:
            クエリ結果のリスト

        Raises:
            DatabaseException: クエリ実行に失敗した場合
        """
        return self.execute_query(query, parameters)

    @contextmanager
    def transaction(self) -> Generator["Neo4jClient", None, None]:
        """
        トランザクションコンテキストマネージャー

        Usage:
            with client.transaction():
                client.execute("CREATE (n:Person {name: $name})", {"name": "太郎"})

        Yields:
            Neo4jClient: 自身のインスタンス
        """
        self.begin_transaction()
        try:
            yield self
            self.commit()
        except Exception:
            self.rollback()
            raise

    def clear_database(self) -> None:
        """
        データベースの全データを削除（テスト用）

        Warning:
            本番環境では使用しないこと
        """
        self.logger.warning("Clearing all data from database")
        self.execute("MATCH (n) DETACH DELETE n")

    def create_constraints(self) -> None:
        """
        制約とインデックスを作成
        """
        constraints = [
            # Personノードのname属性にユニーク制約
            "CREATE CONSTRAINT person_name_unique IF NOT EXISTS FOR (p:Person) REQUIRE p.name IS UNIQUE",

            # Personノードのnameにインデックス
            "CREATE INDEX person_name_index IF NOT EXISTS FOR (p:Person) ON (p.name)",

            # is_decedentフラグにインデックス
            "CREATE INDEX person_decedent_index IF NOT EXISTS FOR (p:Person) ON (p.is_decedent)",

            # is_aliveフラグにインデックス
            "CREATE INDEX person_alive_index IF NOT EXISTS FOR (p:Person) ON (p.is_alive)",
        ]

        for constraint in constraints:
            try:
                self.execute(constraint)
                self.logger.debug(f"Constraint/Index created: {constraint[:50]}...")
            except Exception as e:
                # 制約が既に存在する場合は無視
                self.logger.debug(f"Constraint/Index already exists or failed: {e}")

    def __enter__(self) -> "Neo4jClient":
        """コンテキストマネージャーのエントリー"""
        self.connect()
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """コンテキストマネージャーのイグジット"""
        self.disconnect()



"""日本の元号と西暦の変換ユーティリティ

このモジュールは日本の元号（明治、大正、昭和、平成、令和）と
西暦の相互変換機能を提供します。
"""

from datetime import date
from typing import Dict, Optional, Tuple
import re


# 元号マッピングデータ
# Format: 元号名 -> (略称, 開始年, 開始月, 開始日, 終了年, 終了月, 終了日)
ERA_MAP: Dict[str, Tuple[str, int, int, int, Optional[int], Optional[int], Optional[int]]] = {
    "明治": ("M", 1868, 1, 25, 1912, 7, 30),
    "大正": ("T", 1912, 7, 30, 1926, 12, 25),
    "昭和": ("S", 1926, 12, 25, 1989, 1, 7),
    "平成": ("H", 1989, 1, 8, 2019, 4, 30),
    "令和": ("R", 2019, 5, 1, None, None, None),
}

# 略称から元号名へのマッピング
ABBREV_TO_ERA: Dict[str, str] = {
    "M": "明治",
    "T": "大正",
    "S": "昭和",
    "H": "平成",
    "R": "令和",
}


class EraConversionError(ValueError):
    """元号変換エラー"""
    pass


def parse_japanese_date(input_str: str) -> date:
    """元号形式の日付を西暦dateオブジェクトに変換

    Args:
        input_str: 元号形式の日付文字列
                  例: "令和5年10月3日", "R5.10.3", "R5/10/3"

    Returns:
        date: 西暦のdateオブジェクト

    Raises:
        EraConversionError: 変換できない形式の場合

    Examples:
        >>> parse_japanese_date("令和5年10月3日")
        datetime.date(2023, 10, 3)
        >>> parse_japanese_date("R5.10.3")
        datetime.date(2023, 10, 3)
        >>> parse_japanese_date("H31/4/30")
        datetime.date(2019, 4, 30)
    """
    # 全角数字を半角に変換
    input_str = _normalize_numbers(input_str)

    # パターン1: 令和5年10月3日
    pattern1 = r"^([明大昭平令][治正和成和])(\d{1,2})年(\d{1,2})月(\d{1,2})日$"
    match = re.match(pattern1, input_str)
    if match:
        era_name = match.group(1)
        era_year = int(match.group(2))
        month = int(match.group(3))
        day = int(match.group(4))
        return _convert_era_to_date(era_name, era_year, month, day)

    # パターン2: R5.10.3 または R5/10/3
    pattern2 = r"^([MTSHR])(\d{1,2})[./](\d{1,2})[./](\d{1,2})$"
    match = re.match(pattern2, input_str)
    if match:
        abbrev = match.group(1)
        era_year = int(match.group(2))
        month = int(match.group(3))
        day = int(match.group(4))

        if abbrev not in ABBREV_TO_ERA:
            raise EraConversionError(f"不明な元号略称: {abbrev}")

        era_name = ABBREV_TO_ERA[abbrev]
        return _convert_era_to_date(era_name, era_year, month, day)

    # パターン3: 西暦形式（YYYY-MM-DD, YYYY/MM/DD, YYYY.MM.DD）
    pattern3 = r"^(\d{4})[\-/.](\d{1,2})[\-/.](\d{1,2})$"
    match = re.match(pattern3, input_str)
    if match:
        year = int(match.group(1))
        month = int(match.group(2))
        day = int(match.group(3))
        try:
            return date(year, month, day)
        except ValueError as e:
            raise EraConversionError(f"無効な日付: {input_str}") from e

    raise EraConversionError(
        f"サポートされていない日付形式: {input_str}\n"
        f"サポート形式: 令和5年10月3日, R5.10.3, R5/10/3, 2023-10-03"
    )


def _normalize_numbers(text: str) -> str:
    """全角数字を半角数字に変換"""
    zen_to_han = str.maketrans("０１２３４５６７８９", "0123456789")
    return text.translate(zen_to_han)


def _convert_era_to_date(era_name: str, era_year: int, month: int, day: int) -> date:
    """元号と年月日を西暦dateに変換

    Args:
        era_name: 元号名（例: "令和"）
        era_year: 元号の年（例: 5）
        month: 月
        day: 日

    Returns:
        date: 西暦のdateオブジェクト

    Raises:
        EraConversionError: 無効な元号や日付の場合
    """
    if era_name not in ERA_MAP:
        raise EraConversionError(f"不明な元号: {era_name}")

    abbrev, start_year, start_month, start_day, end_year, end_month, end_day = ERA_MAP[era_name]

    # 元号元年は1年として扱う
    if era_year < 1:
        raise EraConversionError(f"{era_name}の年は1以上である必要があります: {era_year}")

    # 西暦年を計算
    western_year = start_year + era_year - 1

    # 日付を作成
    try:
        result_date = date(western_year, month, day)
    except ValueError as e:
        raise EraConversionError(f"無効な日付: {era_name}{era_year}年{month}月{day}日") from e

    # 元号の開始日より前でないかチェック
    era_start = date(start_year, start_month, start_day)
    if result_date < era_start:
        raise EraConversionError(
            f"{era_name}{era_year}年{month}月{day}日は{era_name}の開始日（{era_start}）より前です"
        )

    # 元号の終了日より後でないかチェック（令和は終了日なし）
    if end_year is not None and end_month is not None and end_day is not None:
        era_end = date(end_year, end_month, end_day)
        if result_date > era_end:
            raise EraConversionError(
                f"{era_name}{era_year}年{month}月{day}日は{era_name}の終了日（{era_end}）より後です"
            )

    return result_date


def format_japanese_date(target_date: date, format_type: str = "long") -> str:
    """西暦dateを元号形式の文字列に変換

    Args:
        target_date: 西暦のdateオブジェクト
        format_type: 出力形式
                    - "long": 令和5年10月3日
                    - "short": R5.10.3
                    - "slash": R5/10/3

    Returns:
        str: 元号形式の日付文字列

    Raises:
        EraConversionError: 明治以前の日付の場合

    Examples:
        >>> format_japanese_date(date(2023, 10, 3), "long")
        '令和5年10月3日'
        >>> format_japanese_date(date(2023, 10, 3), "short")
        'R5.10.3'
    """
    # 該当する元号を検索
    for era_name, (abbrev, start_year, start_month, start_day, end_year, end_month, end_day) in ERA_MAP.items():
        era_start = date(start_year, start_month, start_day)

        # 終了日の判定
        if end_year is not None and end_month is not None and end_day is not None:
            era_end = date(end_year, end_month, end_day)
            if era_start <= target_date <= era_end:
                era_year = target_date.year - start_year + 1
                return _format_era_string(era_name, abbrev, era_year, target_date.month, target_date.day, format_type)
        else:
            # 令和（終了日なし）
            if target_date >= era_start:
                era_year = target_date.year - start_year + 1
                return _format_era_string(era_name, abbrev, era_year, target_date.month, target_date.day, format_type)

    raise EraConversionError(f"明治以前の日付は変換できません: {target_date}")


def _format_era_string(era_name: str, abbrev: str, era_year: int, month: int, day: int, format_type: str) -> str:
    """元号文字列をフォーマット"""
    if format_type == "long":
        return f"{era_name}{era_year}年{month}月{day}日"
    elif format_type == "short":
        return f"{abbrev}{era_year}.{month}.{day}"
    elif format_type == "slash":
        return f"{abbrev}{era_year}/{month}/{day}"
    else:
        raise ValueError(f"不明なフォーマット: {format_type}")


def get_era_name(target_date: date) -> str:
    """指定された日付の元号名を取得

    Args:
        target_date: 西暦のdateオブジェクト

    Returns:
        str: 元号名（例: "令和"）

    Raises:
        EraConversionError: 明治以前の日付の場合
    """
    for era_name, (abbrev, start_year, start_month, start_day, end_year, end_month, end_day) in ERA_MAP.items():
        era_start = date(start_year, start_month, start_day)

        if end_year is not None and end_month is not None and end_day is not None:
            era_end = date(end_year, end_month, end_day)
            if era_start <= target_date <= era_end:
                return era_name
        else:
            if target_date >= era_start:
                return era_name

    raise EraConversionError(f"明治以前の日付は対応していません: {target_date}")



"""Inheritanceモデルのテスト"""
from fractions import Fraction
import pytest
from pydantic import ValidationError

from inheritance_calculator_core.models.person import Person
from inheritance_calculator_core.models.inheritance import (
    Heir,
    InheritanceResult,
    HeritageRank,
    SubstitutionType,
)


class TestHeir:
    """Heirモデルのテスト"""

    def test_create_heir_spouse(self):
        """配偶者相続人の作成"""
        person = Person(name="配偶者")

        heir = Heir(
            person=person,
            rank=HeritageRank.SPOUSE,
            share=Fraction(1, 2),
            share_percentage=50.0
        )

        assert heir.person.name == "配偶者"
        assert heir.rank == HeritageRank.SPOUSE
        assert heir.share == Fraction(1, 2)
        assert heir.share_percentage == 50.0
        assert heir.is_substitution is False
        assert heir.substitution_type == SubstitutionType.NONE

    def test_create_heir_child(self):
        """子相続人の作成"""
        person = Person(name="子")

        heir = Heir(
            person=person,
            rank=HeritageRank.FIRST,
            share=Fraction(1, 4),
            share_percentage=25.0
        )

        assert heir.rank == HeritageRank.FIRST
        assert heir.share == Fraction(1, 4)
        assert heir.share_percentage == 25.0

    def test_create_heir_with_substitution(self):
        """代襲相続人の作成"""
        person = Person(name="孫")
        substituted = Person(name="子", is_alive=False)

        heir = Heir(
            person=person,
            rank=HeritageRank.FIRST,
            share=Fraction(1, 4),
            share_percentage=25.0,
            is_substitution=True,
            substitution_type=SubstitutionType.CHILD,
            substituted_person=substituted,
            generation=2
        )

        assert heir.is_substitution is True
        assert heir.substitution_type == SubstitutionType.CHILD
        assert heir.substituted_person.name == "子"
        assert heir.generation == 2

    def test_share_percentage_auto_calculation(self):
        """相続割合の百分率自動計算"""
        person = Person(name="相続人")

        heir = Heir(
            person=person,
            rank=HeritageRank.FIRST,
            share=Fraction(1, 3),
            share_percentage=0.0  # ダミー値、自動計算される
        )

        # share_percentageはshareから自動計算される
        assert abs(heir.share_percentage - 33.333333) < 0.01

    def test_non_substitution_must_have_none_type(self):
        """代襲相続でない場合、タイプはNONEでなければならない"""
        person = Person(name="相続人")

        with pytest.raises(ValidationError) as exc_info:
            Heir(
                person=person,
                rank=HeritageRank.FIRST,
                share=Fraction(1, 2),
                share_percentage=50.0,
                is_substitution=False,
                substitution_type=SubstitutionType.CHILD
            )
        assert "Non-substitution heir must have NONE type" in str(exc_info.value)

    def test_substitution_must_have_valid_type(self):
        """代襲相続の場合、タイプはNONE以外でなければならない"""
        person = Person(name="孫")

        with pytest.raises(ValidationError) as exc_info:
            Heir(
                person=person,
                rank=HeritageRank.FIRST,
                share=Fraction(1, 4),
                share_percentage=25.0,
                is_substitution=True,
                substitution_type=SubstitutionType.NONE
            )
        assert "Substitution heir must have valid substitution type" in str(exc_info.value)

    def test_heir_string_representation(self):
        """相続人の文字列表現"""
        person = Person(name="山田太郎")
        heir = Heir(
            person=person,
            rank=HeritageRank.FIRST,
            share=Fraction(1, 2),
            share_percentage=50.0
        )

        result = str(heir)
        assert "山田太郎" in result
        assert "first" in result
        assert "1/2" in result
        assert "50.00%" in result

    def test_heir_string_with_substitution(self):
        """代襲相続人の文字列表現"""
        person = Person(name="孫")
        substituted = Person(name="子")

        heir = Heir(
            person=person,
            rank=HeritageRank.FIRST,
            share=Fraction(1, 4),
            share_percentage=25.0,
            is_substitution=True,
            substitution_type=SubstitutionType.CHILD,
            substituted_person=substituted
        )

        result = str(heir)
        assert "孫" in result
        assert "代襲" in result
        assert "子" in result


class TestInheritanceResult:
    """InheritanceResultモデルのテスト"""

    def test_create_empty_result(self):
        """空の相続結果の作成"""
        decedent = Person(name="被相続人", is_decedent=True)

        result = InheritanceResult(decedent=decedent)

        assert result.decedent.name == "被相続人"
        assert len(result.heirs) == 0
        assert result.has_spouse is False
        assert result.has_children is False
        assert result.has_parents is False
        assert result.has_siblings is False

    def test_add_heir(self):
        """相続人追加メソッドのテスト"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        spouse = Person(name="配偶者")
        result.add_heir(
            person=spouse,
            rank=HeritageRank.SPOUSE,
            share=Fraction(1, 1)
        )

        assert len(result.heirs) == 1
        assert result.heirs[0].person.name == "配偶者"
        assert result.heirs[0].share == Fraction(1, 1)

    def test_add_calculation_basis(self):
        """計算根拠追加メソッドのテスト"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        result.add_calculation_basis("民法890条")
        result.add_calculation_basis("民法900条1号")

        assert len(result.calculation_basis) == 2
        assert "民法890条" in result.calculation_basis
        assert "民法900条1号" in result.calculation_basis

    def test_get_heirs_by_rank(self):
        """順位別相続人取得メソッドのテスト"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        spouse = Person(name="配偶者")
        child1 = Person(name="子1")
        child2 = Person(name="子2")

        result.add_heir(spouse, HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(child1, HeritageRank.FIRST, Fraction(1, 4))
        result.add_heir(child2, HeritageRank.FIRST, Fraction(1, 4))

        spouses = result.get_heirs_by_rank(HeritageRank.SPOUSE)
        children = result.get_heirs_by_rank(HeritageRank.FIRST)

        assert len(spouses) == 1
        assert len(children) == 2
        assert spouses[0].person.name == "配偶者"

    def test_get_substitution_heirs(self):
        """代襲相続人取得メソッドのテスト"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        child = Person(name="子")
        grandchild = Person(name="孫")
        substituted = Person(name="死亡した子", is_alive=False)

        result.add_heir(child, HeritageRank.FIRST, Fraction(1, 2))
        result.add_heir(
            grandchild,
            HeritageRank.FIRST,
            Fraction(1, 2),
            is_substitution=True,
            substitution_type=SubstitutionType.CHILD,
            substituted_person=substituted
        )

        substitution_heirs = result.get_substitution_heirs()

        assert len(substitution_heirs) == 1
        assert substitution_heirs[0].person.name == "孫"
        assert substitution_heirs[0].is_substitution is True

    def test_total_heirs_property(self):
        """相続人総数プロパティのテスト"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        assert result.total_heirs == 0

        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(Person(name="子"), HeritageRank.FIRST, Fraction(1, 2))

        assert result.total_heirs == 2

    def test_total_share_validation_success(self):
        """相続割合合計が100%の場合は成功"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(Person(name="子1"), HeritageRank.FIRST, Fraction(1, 4))
        result.add_heir(Person(name="子2"), HeritageRank.FIRST, Fraction(1, 4))

        # バリデーションが通ればOK
        assert result.total_heirs == 3

    def test_total_share_validation_failure(self):
        """相続割合合計が100%でない場合はエラー"""
        decedent = Person(name="被相続人", is_decedent=True)

        with pytest.raises(ValidationError) as exc_info:
            InheritanceResult(
                decedent=decedent,
                heirs=[
                    Heir(
                        person=Person(name="配偶者"),
                        rank=HeritageRank.SPOUSE,
                        share=Fraction(1, 2),
                        share_percentage=50.0
                    ),
                    Heir(
                        person=Person(name="子"),
                        rank=HeritageRank.FIRST,
                        share=Fraction(1, 3),  # 合計が100%にならない
                        share_percentage=33.33
                    )
                ]
            )
        assert "Total share must be 1 (100%)" in str(exc_info.value)

    def test_string_representation(self):
        """相続結果の文字列表現"""
        decedent = Person(name="山田太郎", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        result.add_heir(Person(name="山田花子"), HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(Person(name="山田一郎"), HeritageRank.FIRST, Fraction(1, 2))
        result.add_calculation_basis("民法890条")

        output = str(result)

        assert "山田太郎" in output
        assert "山田花子" in output
        assert "山田一郎" in output
        assert "相続人総数: 2名" in output
        assert "民法890条" in output


class TestComplexInheritanceScenarios:
    """複雑な相続シナリオのテスト"""

    def test_spouse_and_children(self):
        """配偶者と子のケース"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        # 配偶者: 1/2, 子2人: 各1/4
        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(Person(name="子1"), HeritageRank.FIRST, Fraction(1, 4))
        result.add_heir(Person(name="子2"), HeritageRank.FIRST, Fraction(1, 4))

        assert result.total_heirs == 3
        total = sum(heir.share for heir in result.heirs)
        assert total == Fraction(1, 1)

    def test_spouse_and_parents(self):
        """配偶者と直系尊属のケース"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        # 配偶者: 2/3, 父母: 各1/6
        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(2, 3))
        result.add_heir(Person(name="父"), HeritageRank.SECOND, Fraction(1, 6))
        result.add_heir(Person(name="母"), HeritageRank.SECOND, Fraction(1, 6))

        assert result.total_heirs == 3
        total = sum(heir.share for heir in result.heirs)
        assert total == Fraction(1, 1)

    def test_spouse_and_siblings(self):
        """配偶者と兄弟姉妹のケース"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        # 配偶者: 3/4, 兄弟姉妹2人: 各1/8
        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(3, 4))
        result.add_heir(Person(name="兄"), HeritageRank.THIRD, Fraction(1, 8))
        result.add_heir(Person(name="妹"), HeritageRank.THIRD, Fraction(1, 8))

        assert result.total_heirs == 3
        total = sum(heir.share for heir in result.heirs)
        assert total == Fraction(1, 1)

    def test_child_substitution(self):
        """子の代襲相続のケース"""
        decedent = Person(name="被相続人", is_decedent=True)
        result = InheritanceResult(decedent=decedent)

        substituted = Person(name="死亡した子", is_alive=False)

        # 配偶者: 1/2, 存命の子: 1/4, 孫(代襲): 1/4
        result.add_heir(Person(name="配偶者"), HeritageRank.SPOUSE, Fraction(1, 2))
        result.add_heir(Person(name="存命の子"), HeritageRank.FIRST, Fraction(1, 4))
        result.add_heir(
            Person(name="孫"),
            HeritageRank.FIRST,
            Fraction(1, 4),
            is_substitution=True,
            substitution_type=SubstitutionType.CHILD,
            substituted_person=substituted
        )

        substitution_heirs = result.get_substitution_heirs()
        assert len(substitution_heirs) == 1
        assert substitution_heirs[0].person.name == "孫"

        total = sum(heir.share for heir in result.heirs)
        assert total == Fraction(1, 1)



"""Neo4jクライアントのテスト"""
import pytest
from datetime import date
from unittest.mock import patch

from inheritance_calculator_core.database.neo4j_client import Neo4jClient
from inheritance_calculator_core.database.repositories import PersonRepository
from inheritance_calculator_core.models.person import Person
from inheritance_calculator_core.utils.exceptions import DatabaseException


class TestNeo4jClient:
    """Neo4jクライアントのテスト（モック使用）"""

    def test_connection(self, neo4j_client_mock):
        """接続テスト"""
        assert neo4j_client_mock.is_connected()

    def test_health_check(self, neo4j_client_mock):
        """ヘルスチェックのテスト"""
        assert neo4j_client_mock.health_check()

    def test_execute_simple_query(self, neo4j_client_mock):
        """単純なクエリ実行のテスト"""
        result = neo4j_client_mock.execute("RETURN 1 as value")
        assert len(result) == 1
        assert result[0]["value"] == 1

    def test_transaction(self, neo4j_client_mock):
        """トランザクションのテスト"""
        with neo4j_client_mock.transaction():
            neo4j_client_mock.execute(
                "CREATE (p:Person {name: $name})",
                {"name": "テスト太郎"}
            )

        # トランザクション外で確認
        result = neo4j_client_mock.execute(
            "MATCH (p:Person {name: $name}) RETURN p",
            {"name": "テスト太郎"}
        )
        assert len(result) == 1


class TestPersonRepository:
    """PersonRepositoryのテスト（モック使用）"""

    @pytest.mark.skip(reason="PersonRepositoryはリアルなNeo4jクエリロジックが必要")
    def test_create_person(self, person_repo_mock):
        """人物作成のテスト"""
        person = Person(
            name="山田太郎",
            is_alive=False,
            is_decedent=True,
            birth_date=date(1950, 1, 1),
            death_date=date(2025, 6, 15)
        )

        created = person_repo_mock.create(person)
        assert created.name == "山田太郎"

    @pytest.mark.skip(reason="PersonRepositoryはリアルなNeo4jクエリロジックが必要")
    def test_find_by_name(self, person_repo_mock):
        """名前検索のテスト"""
        person = Person(
            name="山田花子",
            is_alive=True,
            birth_date=date(1955, 3, 10)
        )
        person_repo_mock.create(person)

        found = person_repo_mock.find_by_name("山田花子")
        assert found is not None
        assert found.name == "山田花子"
        assert found.is_alive is True

    @pytest.mark.skip(reason="PersonRepositoryはリアルなNeo4jクエリロジックが必要")
    def test_find_decedent(self, person_repo_mock):
        """被相続人検索のテスト"""
        decedent = Person(
            name="被相続人",
            is_alive=False,
            is_decedent=True,
            death_date=date(2025, 6, 15)
        )
        person_repo_mock.create(decedent)

        found = person_repo_mock.find_decedent()
        assert found is not None
        assert found.name == "被相続人"
        assert found.is_decedent is True

    @pytest.mark.skip(reason="PersonRepositoryはリアルなNeo4jクエリロジックが必要")
    def test_find_all(self, person_repo_mock):
        """全人物取得のテスト"""
        person1 = Person(name="太郎", is_alive=True)
        person2 = Person(name="花子", is_alive=True)

        person_repo_mock.create(person1)
        person_repo_mock.create(person2)

        all_persons = person_repo_mock.find_all()
        assert len(all_persons) == 2

    @pytest.mark.skip(reason="PersonRepositoryはリアルなNeo4jクエリロジックが必要")
    def test_delete(self, person_repo_mock):
        """削除のテスト"""
        person = Person(name="削除テスト", is_alive=True)
        person_repo_mock.create(person)

        person_repo_mock.delete("削除テスト")

        found = person_repo_mock.find_by_name("削除テスト")
        assert found is None


def _run_integration_enabled(config):
    """統合テストが有効かチェック"""
    return config.getoption("--run-integration", default=False)


@pytest.mark.skipif(
    "not config.getoption('--run-integration', default=False)",
    reason="Integration tests require Neo4j running"
)
class TestIntegration:
    """統合テスト（Neo4j実行環境が必要）"""

    def test_full_workflow(self, person_repo, neo4j_client):
        """完全なワークフローのテスト"""
        from inheritance_calculator_core.database.repositories import RelationshipRepository

        # リポジトリ作成
        rel_repo = RelationshipRepository(neo4j_client)

        # 被相続人作成
        decedent = Person(
            name="被相続人太郎",
            is_alive=False,
            is_decedent=True,
            death_date=date(2025, 6, 15)
        )
        person_repo.create(decedent)

        # 配偶者作成
        spouse = Person(name="配偶者花子", is_alive=True)
        person_repo.create(spouse)
        rel_repo.create_spouse_of("被相続人太郎", "配偶者花子")

        # 子作成
        child = Person(name="子一郎", is_alive=True)
        person_repo.create(child)
        rel_repo.create_child_of("子一郎", "被相続人太郎")

        # リレーションシップの確認
        children = rel_repo.find_children("被相続人太郎")
        assert len(children) == 1
        assert children[0].name == "子一郎"

        found_spouse = rel_repo.find_spouse("被相続人太郎")
        assert found_spouse is not None
        assert found_spouse.name == "配偶者花子"


def pytest_addoption(parser):
    """pytestオプションの追加"""
    parser.addoption(
        "--run-integration",
        action="store_true",
        default=False,
        help="Run integration tests requiring Neo4j"
    )



"""Neo4jサービス

相続計算結果をNeo4jに保存するサービス。
"""
from typing import List, Dict
from datetime import date
import logging

from ..database.neo4j_client import Neo4jClient
from ..database.repositories import PersonRepository, RelationshipRepository
from ..models.person import Person
from ..models.relationship import BloodType
from ..models.inheritance import InheritanceResult
from ..models.value_objects import PersonID
from ..utils.exceptions import DatabaseException


class Neo4jService:
    """
    Neo4jデータ保存サービス

    相続計算に使用した人物情報と関係性をNeo4jに保存する。
    """

    def __init__(self, client: Neo4jClient) -> None:
        """
        初期化

        Args:
            client: Neo4jクライアント
        """
        self.client = client
        self.person_repo = PersonRepository(client)
        self.relationship_repo = RelationshipRepository(client)
        self.logger = logging.getLogger(__name__)

    def save_inheritance_case(
        self,
        decedent: Person,
        spouses: List[Person],
        children: List[Person],
        parents: List[Person],
        siblings: List[Person],
        renounced: List[Person],
        disqualified: List[Person],
        disinherited: List[Person],
        sibling_blood_types: Dict[PersonID, BloodType],
        result: InheritanceResult
    ) -> None:
        """
        相続ケースをNeo4jに保存

        Args:
            decedent: 被相続人
            spouses: 配偶者リスト
            children: 子リスト
            parents: 直系尊属リスト
            siblings: 兄弟姉妹リスト
            renounced: 相続放棄者リスト
            disqualified: 相続欠格者リスト
            disinherited: 相続廃除者リスト
            sibling_blood_types: 兄弟姉妹の血縁タイプ
            result: 計算結果

        Raises:
            DatabaseException: 保存に失敗した場合
        """
        try:
            with self.client.transaction():
                # 被相続人を保存
                self.logger.info(f"Saving decedent: {decedent.name}")
                self.person_repo.create(decedent)

                # 配偶者を保存
                for spouse in spouses:
                    self.logger.info(f"Saving spouse: {spouse.name}")
                    self.person_repo.create(spouse)
                    self.relationship_repo.create_spouse_of(
                        person1_name=decedent.name,
                        person2_name=spouse.name,
                        is_current=True
                    )

                # 子を保存
                for child in children:
                    self.logger.info(f"Saving child: {child.name}")
                    self.person_repo.create(child)
                    self.relationship_repo.create_child_of(
                        child_name=child.name,
                        parent_name=decedent.name
                    )

                # 直系尊属を保存
                for parent in parents:
                    self.logger.info(f"Saving parent: {parent.name}")
                    self.person_repo.create(parent)
                    self.relationship_repo.create_child_of(
                        child_name=decedent.name,
                        parent_name=parent.name
                    )

                # 兄弟姉妹を保存
                for sibling in siblings:
                    self.logger.info(f"Saving sibling: {sibling.name}")
                    self.person_repo.create(sibling)
                    blood_type = sibling_blood_types.get(sibling.id, BloodType.FULL)
                    self.relationship_repo.create_sibling_of(
                        person1_name=decedent.name,
                        person2_name=sibling.name,
                        blood_type=blood_type
                    )

                # 相続放棄者を保存
                for person in renounced:
                    self.logger.info(f"Saving renounced: {person.name}")
                    # 既に人物ノードが存在する可能性があるため、存在チェック
                    if not self.person_repo.find_by_name(person.name):
                        self.person_repo.create(person)

                    # 放棄日は被相続人の死亡日、またはデフォルト値を使用
                    renounce_date = decedent.death_date if decedent.death_date is not None else date.today()

                    self.relationship_repo.create_renounced(
                        person_name=person.name,
                        decedent_name=decedent.name,
                        renounce_date=renounce_date
                    )

                self.logger.info("Successfully saved inheritance case to Neo4j")

        except Exception as e:
            self.logger.error(f"Failed to save inheritance case: {e}", exc_info=True)
            raise DatabaseException(f"Neo4j保存エラー: {str(e)}")

    def clear_all_data(self) -> None:
        """
        全データを削除（テスト用）

        Warning:
            本番環境では使用しないこと
        """
        self.logger.warning("Clearing all data from Neo4j")
        self.client.clear_database()



"""データベーステスト用の共通フィクスチャとモック"""
import pytest
from unittest.mock import MagicMock, Mock, patch
from typing import Any, Dict, List, Optional

from inheritance_calculator_core.database.neo4j_client import Neo4jClient
from inheritance_calculator_core.database.repositories import PersonRepository


class MockNeo4jDriver:
    """Neo4jドライバーのモック"""

    def __init__(self):
        self.connected = True
        self.sessions = []
        # 全セッションで共有するデータストア
        self.shared_data: Dict[str, Any] = {}

    def verify_connectivity(self):
        """接続確認（モック）"""
        if not self.connected:
            raise Exception("Not connected")

    def session(self, database: Optional[str] = None):
        """セッション作成（モック）"""
        session = MockNeo4jSession(self.shared_data)
        self.sessions.append(session)
        return session

    def close(self):
        """ドライバーを閉じる"""
        self.connected = False


class MockNeo4jSession:
    """Neo4jセッションのモック"""

    def __init__(self, shared_data: Dict[str, Any]):
        self.closed = False
        self.transaction_active = False
        # 共有データストアへの参照
        self.data: Dict[str, Any] = shared_data
        self.current_transaction: Optional['MockTransaction'] = None

    def run(self, query: str, parameters: Optional[Dict[str, Any]] = None):
        """クエリ実行（モック）"""
        # 単純な健全性チェッククエリ
        if query == "RETURN 1 as health":
            return MockResult([{"health": 1}])

        # 単純な値返却クエリ
        if query == "RETURN 1 as value":
            return MockResult([{"value": 1}])

        # CREATE クエリ
        if "CREATE" in query and "Person" in query:
            params = parameters or {}
            name = params.get("name", "unknown")
            self.data[name] = params
            return MockResult([])

        # MATCH クエリ
        if "MATCH" in query and "Person" in query:
            params = parameters or {}
            name = params.get("name")
            if name and name in self.data:
                return MockResult([{"p": self.data[name]}])
            return MockResult([])

        # DETACH DELETE クエリ（データベースクリア）
        if "DETACH DELETE" in query:
            self.data.clear()
            return MockResult([])

        # 制約作成クエリ
        if "CREATE CONSTRAINT" in query or "CREATE INDEX" in query:
            return MockResult([])

        # デフォルトは空の結果
        return MockResult([])

    def begin_transaction(self):
        """トランザクション開始（モック）"""
        self.transaction_active = True
        self.current_transaction = MockTransaction(self)
        return self.current_transaction

    def close(self):
        """セッションを閉じる"""
        self.closed = True

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


class MockTransaction:
    """Neo4jトランザクションのモック"""

    def __init__(self, session: MockNeo4jSession):
        self.session = session
        self.committed = False
        self.rolled_back = False
        self.pending_data: Dict[str, Any] = {}

    def run(self, query: str, parameters: Optional[Dict[str, Any]] = None):
        """トランザクション内でのクエリ実行（モック）"""
        # CREATE クエリの場合、pending_dataに保存
        if "CREATE" in query and "Person" in query:
            params = parameters or {}
            name = params.get("name", "unknown")
            self.pending_data[name] = params
            return MockResult([])

        # MATCH クエリの場合、pending_dataとcommittedデータの両方を確認
        if "MATCH" in query and "Person" in query:
            params = parameters or {}
            name = params.get("name")
            if name:
                # pending_dataを優先的に確認
                if name in self.pending_data:
                    return MockResult([{"p": self.pending_data[name]}])
                # session.dataも確認
                if name in self.session.data:
                    return MockResult([{"p": self.session.data[name]}])
            return MockResult([])

        # その他のクエリはsessionに委譲
        return self.session.run(query, parameters)

    def commit(self):
        """コミット"""
        self.committed = True
        # pending_dataをsession.dataにマージ
        self.session.data.update(self.pending_data)
        self.pending_data.clear()
        self.session.transaction_active = False
        self.session.current_transaction = None

    def rollback(self):
        """ロールバック"""
        self.rolled_back = True
        # pending_dataを破棄
        self.pending_data.clear()
        self.session.transaction_active = False
        self.session.current_transaction = None


class MockResult:
    """Neo4jクエリ結果のモック"""

    def __init__(self, records: List[Dict[str, Any]]):
        self.records = records
        self._index = 0

    def single(self):
        """単一レコードの取得"""
        if self.records:
            return self.records[0]
        return None

    def __iter__(self):
        return iter(self.records)

    def __next__(self):
        if self._index < len(self.records):
            record = self.records[self._index]
            self._index += 1
            return record
        raise StopIteration


@pytest.fixture
def mock_neo4j_driver():
    """モックNeo4jドライバーのフィクスチャ"""
    return MockNeo4jDriver()


@pytest.fixture
def neo4j_client_mock(mock_neo4j_driver):
    """モックを使用したNeo4jクライアントのフィクスチャ"""
    with patch('src.database.neo4j_client.GraphDatabase.driver', return_value=mock_neo4j_driver):
        client = Neo4jClient()
        client.connect()
        yield client
        if client.is_connected():
            client.disconnect()


@pytest.fixture
def person_repo_mock(neo4j_client_mock):
    """モックを使用したPersonRepositoryのフィクスチャ"""
    return PersonRepository(neo4j_client_mock)



"""相続割合計算サービス

日本の民法に基づいて相続割合（法定相続分）を計算するサービス。
"""
from typing import List, Dict, Optional
from fractions import Fraction

from ..models.person import Person
from ..models.relationship import BloodType
from ..models.inheritance import HeritageRank
from ..models.value_objects import PersonID
from .base import BaseService


class ShareCalculator(BaseService[Person]):
    """
    相続割合計算サービス

    法定相続分を計算する。
    """

    def calculate_shares(
        self,
        spouses: List[Person],
        first_rank: List[Person],
        second_rank: List[Person],
        third_rank: List[Person],
        third_rank_blood_types: Optional[Dict[PersonID, BloodType]] = None
    ) -> Dict[PersonID, Fraction]:
        """
        相続割合を計算

        Args:
            spouses: 配偶者のリスト
            first_rank: 第1順位の相続人（子）
            second_rank: 第2順位の相続人（直系尊属）
            third_rank: 第3順位の相続人（兄弟姉妹）
            third_rank_blood_types: 第3順位の血縁タイプ（PersonID → BloodType）

        Returns:
            PersonID → 相続割合の辞書
        """
        if third_rank_blood_types is None:
            third_rank_blood_types = {}

        shares: Dict[PersonID, Fraction] = {}

        # 配偶者のみの場合
        if spouses and not first_rank and not second_rank and not third_rank:
            shares.update(self._calculate_spouse_only(spouses))
            self.log_operation(
                "Calculated shares: spouse only",
                basis="配偶者のみ（全部相続）"
            )
            return shares

        # 配偶者と子
        if spouses and first_rank:
            shares.update(self._calculate_spouse_and_children(spouses, first_rank))
            self.log_operation(
                "Calculated shares: spouse and children",
                basis="民法900条1号"
            )
            return shares

        # 子のみ
        if first_rank and not spouses:
            shares.update(self._calculate_children_only(first_rank))
            self.log_operation(
                "Calculated shares: children only",
                basis="子のみ（均等分割）"
            )
            return shares

        # 配偶者と直系尊属
        if spouses and second_rank:
            shares.update(self._calculate_spouse_and_parents(spouses, second_rank))
            self.log_operation(
                "Calculated shares: spouse and parents",
                basis="民法900条2号"
            )
            return shares

        # 直系尊属のみ
        if second_rank and not spouses:
            shares.update(self._calculate_parents_only(second_rank))
            self.log_operation(
                "Calculated shares: parents only",
                basis="直系尊属のみ（均等分割）"
            )
            return shares

        # 配偶者と兄弟姉妹
        if spouses and third_rank:
            shares.update(
                self._calculate_spouse_and_siblings(
                    spouses, third_rank, third_rank_blood_types
                )
            )
            self.log_operation(
                "Calculated shares: spouse and siblings",
                basis="民法900条3号・4号"
            )
            return shares

        # 兄弟姉妹のみ
        if third_rank and not spouses:
            shares.update(
                self._calculate_siblings_only(third_rank, third_rank_blood_types)
            )
            self.log_operation(
                "Calculated shares: siblings only",
                basis="兄弟姉妹のみ（民法900条4号）"
            )
            return shares

        # 相続人がいない場合
        self.log_warning("No heirs found")
        return shares

    def _calculate_spouse_only(self, spouses: List[Person]) -> Dict[PersonID, Fraction]:
        """配偶者のみの場合（全部相続）"""
        shares = {}
        share_per_spouse = Fraction(1, len(spouses))
        for spouse in spouses:
            shares[spouse.id] = share_per_spouse
        return shares

    def _calculate_spouse_and_children(
        self, spouses: List[Person], children: List[Person]
    ) -> Dict[PersonID, Fraction]:
        """
        配偶者と子の場合

        民法900条1号：
        - 配偶者: 1/2
        - 子: 1/2を均等に分割
        """
        shares = {}

        # 配偶者: 1/2
        spouse_total = Fraction(1, 2)
        share_per_spouse = spouse_total / len(spouses)
        for spouse in spouses:
            shares[spouse.id] = share_per_spouse

        # 子: 1/2を均等に分割
        children_total = Fraction(1, 2)
        share_per_child = children_total / len(children)
        for child in children:
            shares[child.id] = share_per_child

        return shares

    def _calculate_children_only(self, children: List[Person]) -> Dict[PersonID, Fraction]:
        """子のみの場合（均等分割）"""
        shares = {}
        share_per_child = Fraction(1, len(children))
        for child in children:
            shares[child.id] = share_per_child
        return shares

    def _calculate_spouse_and_parents(
        self, spouses: List[Person], parents: List[Person]
    ) -> Dict[PersonID, Fraction]:
        """
        配偶者と直系尊属の場合

        民法900条2号：
        - 配偶者: 2/3
        - 直系尊属: 1/3を均等に分割
        """
        shares = {}

        # 配偶者: 2/3
        spouse_total = Fraction(2, 3)
        share_per_spouse = spouse_total / len(spouses)
        for spouse in spouses:
            shares[spouse.id] = share_per_spouse

        # 直系尊属: 1/3を均等に分割
        parents_total = Fraction(1, 3)
        share_per_parent = parents_total / len(parents)
        for parent in parents:
            shares[parent.id] = share_per_parent

        return shares

    def _calculate_parents_only(self, parents: List[Person]) -> Dict[PersonID, Fraction]:
        """直系尊属のみの場合（均等分割）"""
        shares = {}
        share_per_parent = Fraction(1, len(parents))
        for parent in parents:
            shares[parent.id] = share_per_parent
        return shares

    def _calculate_spouse_and_siblings(
        self,
        spouses: List[Person],
        siblings: List[Person],
        blood_types: Dict[PersonID, BloodType]
    ) -> Dict[PersonID, Fraction]:
        """
        配偶者と兄弟姉妹の場合

        民法900条3号：
        - 配偶者: 3/4
        - 兄弟姉妹: 1/4を分割

        民法900条4号：
        - 半血兄弟姉妹の相続分は全血兄弟姉妹の1/2
        """
        shares = {}

        # 配偶者: 3/4
        spouse_total = Fraction(3, 4)
        share_per_spouse = spouse_total / len(spouses)
        for spouse in spouses:
            shares[spouse.id] = share_per_spouse

        # 兄弟姉妹: 1/4を血縁タイプに応じて分割
        siblings_total = Fraction(1, 4)
        shares.update(
            self._calculate_sibling_shares(siblings, siblings_total, blood_types)
        )

        return shares

    def _calculate_siblings_only(
        self,
        siblings: List[Person],
        blood_types: Dict[PersonID, BloodType]
    ) -> Dict[PersonID, Fraction]:
        """
        兄弟姉妹のみの場合

        民法900条4号：
        - 半血兄弟姉妹の相続分は全血兄弟姉妹の1/2
        """
        shares = {}
        total = Fraction(1, 1)
        shares.update(self._calculate_sibling_shares(siblings, total, blood_types))
        return shares

    def _calculate_sibling_shares(
        self,
        siblings: List[Person],
        total: Fraction,
        blood_types: Dict[PersonID, BloodType]
    ) -> Dict[PersonID, Fraction]:
        """
        兄弟姉妹の相続分を計算

        民法900条4号：半血兄弟姉妹の相続分は全血兄弟姉妹の1/2

        Args:
            siblings: 兄弟姉妹のリスト
            total: 兄弟姉妹全体の相続分
            blood_types: 血縁タイプ（PersonID → BloodType）

        Returns:
            PersonID → 相続割合の辞書
        """
        shares = {}

        # 全血・半血の人数をカウント
        full_blood_count = sum(
            1 for s in siblings
            if blood_types.get(s.id, BloodType.FULL) == BloodType.FULL
        )
        half_blood_count = len(siblings) - full_blood_count

        # 全血を1、半血を0.5として重み付けした総数を計算
        # 全血1人 = 重み1.0、半血1人 = 重み0.5
        total_weight = full_blood_count + (half_blood_count * Fraction(1, 2))

        # 各兄弟姉妹の相続分を計算
        for sibling in siblings:
            blood_type = blood_types.get(sibling.id, BloodType.FULL)

            if blood_type == BloodType.FULL:
                # 全血: total / total_weight
                shares[sibling.id] = total / total_weight
            else:
                # 半血: (total / total_weight) * 1/2
                shares[sibling.id] = (total / total_weight) * Fraction(1, 2)

        return shares



"""リポジトリ層のユニットテスト

PersonRepository, RelationshipRepository, InheritanceRepositoryのテスト
モックを使用してNeo4jに依存しないユニットテストを実装
"""
import pytest
from datetime import date
from unittest.mock import Mock, MagicMock, patch, call
from neo4j.exceptions import Neo4jError

from inheritance_calculator_core.database.repositories import PersonRepository, RelationshipRepository, InheritanceRepository
from inheritance_calculator_core.models.person import Person, Gender
from inheritance_calculator_core.models.relationship import BloodType
from inheritance_calculator_core.utils.exceptions import DatabaseException


class TestPersonRepository:
    """PersonRepositoryのユニットテスト"""

    @pytest.fixture
    def mock_client(self):
        """モッククライアントのフィクスチャ"""
        return Mock()

    @pytest.fixture
    def person_repo(self, mock_client):
        """PersonRepositoryのフィクスチャ"""
        return PersonRepository(mock_client)

    @pytest.fixture
    def sample_person(self):
        """サンプルPersonのフィクスチャ"""
        return Person(
            name="山田太郎",
            is_alive=False,
            is_decedent=True,
            birth_date=date(1950, 1, 1),
            death_date=date(2025, 6, 15),
            gender=Gender.MALE
        )

    def test_create_person_success(self, person_repo, mock_client, sample_person):
        """人物作成の成功テスト"""
        mock_client.execute.return_value = [{"p": {}}]

        result = person_repo.create(sample_person)

        assert result == sample_person
        assert mock_client.execute.call_count == 1
        # クエリパラメータの検証
        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["name"] == "山田太郎"
        assert params["is_alive"] is False
        assert params["is_decedent"] is True

    def test_create_person_without_optional_fields(self, person_repo, mock_client):
        """オプショナルフィールドなしの人物作成テスト"""
        person = Person(
            name="山田花子",
            is_alive=True,
            is_decedent=False
        )
        mock_client.execute.return_value = [{"p": {}}]

        result = person_repo.create(person)

        assert result.name == "山田花子"
        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["birth_date"] is None
        assert params["death_date"] is None

    def test_create_person_failure(self, person_repo, mock_client, sample_person):
        """人物作成の失敗テスト"""
        mock_client.execute.side_effect = Neo4jError("Database error")

        with pytest.raises(DatabaseException) as exc_info:
            person_repo.create(sample_person)

        assert "人物作成エラー" in str(exc_info.value)

    def test_find_by_name_found(self, person_repo, mock_client):
        """名前検索の成功テスト"""
        mock_node = {
            "name": "山田太郎",
            "is_alive": False,
            "is_decedent": True,
            "birth_date": Mock(year=1950, month=1, day=1),
            "death_date": Mock(year=2025, month=6, day=15),
            "gender": "male"
        }
        mock_client.execute.return_value = [{"p": mock_node}]

        result = person_repo.find_by_name("山田太郎")

        assert result is not None
        assert result.name == "山田太郎"
        assert result.is_alive is False
        assert result.is_decedent is True
        assert result.gender == Gender.MALE

    def test_find_by_name_not_found(self, person_repo, mock_client):
        """名前検索で見つからない場合のテスト"""
        mock_client.execute.return_value = []

        result = person_repo.find_by_name("存在しない人")

        assert result is None

    def test_find_by_name_database_error(self, person_repo, mock_client):
        """名前検索のデータベースエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Connection error")

        with pytest.raises(DatabaseException) as exc_info:
            person_repo.find_by_name("山田太郎")

        assert "人物検索エラー" in str(exc_info.value)

    def test_find_decedent_found(self, person_repo, mock_client):
        """被相続人検索の成功テスト"""
        mock_node = {
            "name": "被相続人",
            "is_alive": False,
            "is_decedent": True,
            "birth_date": None,
            "death_date": Mock(year=2025, month=6, day=15),
            "gender": None
        }
        mock_client.execute.return_value = [{"p": mock_node}]

        result = person_repo.find_decedent()

        assert result is not None
        assert result.name == "被相続人"
        assert result.is_decedent is True

    def test_find_decedent_not_found(self, person_repo, mock_client):
        """被相続人が存在しない場合のテスト"""
        mock_client.execute.return_value = []

        result = person_repo.find_decedent()

        assert result is None

    def test_find_all_success(self, person_repo, mock_client):
        """全人物取得の成功テスト"""
        mock_nodes = [
            {"p": {"name": "太郎", "is_alive": True, "is_decedent": False}},
            {"p": {"name": "花子", "is_alive": True, "is_decedent": False}},
            {"p": {"name": "次郎", "is_alive": False, "is_decedent": True,
                   "death_date": Mock(year=2025, month=6, day=15)}}
        ]
        mock_client.execute.return_value = mock_nodes

        result = person_repo.find_all()

        assert len(result) == 3
        assert result[0].name == "太郎"
        assert result[1].name == "花子"
        assert result[2].name == "次郎"

    def test_find_all_empty(self, person_repo, mock_client):
        """全人物取得で結果が空の場合のテスト"""
        mock_client.execute.return_value = []

        result = person_repo.find_all()

        assert len(result) == 0

    def test_update_person_success(self, person_repo, mock_client, sample_person):
        """人物更新の成功テスト"""
        mock_client.execute.return_value = [{"p": {}}]

        result = person_repo.update(sample_person)

        assert result == sample_person
        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["name"] == "山田太郎"
        assert params["is_alive"] is False

    def test_update_person_not_found(self, person_repo, mock_client, sample_person):
        """人物更新で対象が見つからない場合のテスト"""
        mock_client.execute.return_value = []

        with pytest.raises(DatabaseException) as exc_info:
            person_repo.update(sample_person)

        assert "人物が見つかりません" in str(exc_info.value)

    def test_delete_person_success(self, person_repo, mock_client):
        """人物削除の成功テスト"""
        mock_client.execute.return_value = []

        person_repo.delete("山田太郎")

        assert mock_client.execute.call_count == 1

    def test_delete_person_error(self, person_repo, mock_client):
        """人物削除のエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Delete failed")

        with pytest.raises(DatabaseException) as exc_info:
            person_repo.delete("山田太郎")

        assert "人物削除エラー" in str(exc_info.value)

    def test_delete_all_success(self, person_repo, mock_client):
        """全人物削除の成功テスト"""
        mock_client.execute.return_value = []

        person_repo.delete_all()

        assert mock_client.execute.call_count == 1

    def test_to_person_conversion_full_data(self, person_repo):
        """_to_personメソッドの完全データ変換テスト"""
        node = {
            "name": "山田太郎",
            "is_alive": False,
            "is_decedent": True,
            "birth_date": Mock(year=1950, month=1, day=1),
            "death_date": Mock(year=2025, month=6, day=15),
            "gender": "male"
        }

        result = person_repo._to_person(node)

        assert result.name == "山田太郎"
        assert result.is_alive is False
        assert result.is_decedent is True
        assert result.birth_date == date(1950, 1, 1)
        assert result.death_date == date(2025, 6, 15)
        assert result.gender == Gender.MALE

    def test_to_person_conversion_minimal_data(self, person_repo):
        """_to_personメソッドの最小データ変換テスト"""
        node = {
            "name": "山田花子",
            "is_alive": True,
            "is_decedent": False
        }

        result = person_repo._to_person(node)

        assert result.name == "山田花子"
        assert result.is_alive is True
        assert result.birth_date is None
        assert result.death_date is None
        assert result.gender == Gender.UNKNOWN

    def test_create_person_with_contact_info(self, person_repo, mock_client):
        """連絡先情報付き人物作成テスト"""
        person = Person(
            name="山田三郎",
            is_alive=True,
            is_decedent=False,
            address="東京都渋谷区渋谷1-1-1",
            phone="03-1234-5678",
            email="saburo@example.com"
        )
        mock_client.execute.return_value = [{"p": {}}]

        result = person_repo.create(person)

        assert result.name == "山田三郎"
        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["address"] == "東京都渋谷区渋谷1-1-1"
        assert params["phone"] == "03-1234-5678"
        assert params["email"] == "saburo@example.com"

    def test_update_person_with_contact_info(self, person_repo, mock_client):
        """連絡先情報付き人物更新テスト"""
        person = Person(
            name="山田四郎",
            is_alive=True,
            is_decedent=False,
            address="大阪府大阪市北区梅田1-1-1",
            phone="06-9876-5432",
            email="shiro@example.com"
        )
        mock_client.execute.return_value = [{"p": {}}]

        result = person_repo.update(person)

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["address"] == "大阪府大阪市北区梅田1-1-1"
        assert params["phone"] == "06-9876-5432"
        assert params["email"] == "shiro@example.com"

    def test_to_person_conversion_with_contact_info(self, person_repo):
        """連絡先情報を含む_to_personメソッドの変換テスト"""
        node = {
            "name": "山田五郎",
            "is_alive": True,
            "is_decedent": False,
            "address": "愛知県名古屋市中区栄1-1-1",
            "phone": "052-123-4567",
            "email": "goro@example.com"
        }

        result = person_repo._to_person(node)

        assert result.name == "山田五郎"
        assert result.address == "愛知県名古屋市中区栄1-1-1"
        assert result.phone == "052-123-4567"
        assert result.email == "goro@example.com"

    def test_to_person_conversion_partial_contact_info(self, person_repo):
        """部分的な連絡先情報を含む_to_personメソッドの変換テスト"""
        node = {
            "name": "山田六郎",
            "is_alive": True,
            "is_decedent": False,
            "address": "福岡県福岡市博多区博多駅前1-1-1",
            "phone": None,
            "email": None
        }

        result = person_repo._to_person(node)

        assert result.name == "山田六郎"
        assert result.address == "福岡県福岡市博多区博多駅前1-1-1"
        assert result.phone is None
        assert result.email is None


class TestRelationshipRepository:
    """RelationshipRepositoryのユニットテスト"""

    @pytest.fixture
    def mock_client(self):
        """モッククライアントのフィクスチャ"""
        return Mock()

    @pytest.fixture
    def rel_repo(self, mock_client):
        """RelationshipRepositoryのフィクスチャ"""
        return RelationshipRepository(mock_client)

    def test_create_child_of_success(self, rel_repo, mock_client):
        """親子関係作成の成功テスト"""
        mock_client.execute.return_value = []

        rel_repo.create_child_of("子", "親")

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["child_name"] == "子"
        assert params["parent_name"] == "親"
        assert params["adoption"] is False
        assert params["is_biological"] is True

    def test_create_child_of_with_adoption(self, rel_repo, mock_client):
        """養子縁組での親子関係作成テスト"""
        mock_client.execute.return_value = []

        rel_repo.create_child_of("養子", "養親", adoption=True, is_biological=False)

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["adoption"] is True
        assert params["is_biological"] is False

    def test_create_child_of_error(self, rel_repo, mock_client):
        """親子関係作成のエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Constraint violation")

        with pytest.raises(DatabaseException) as exc_info:
            rel_repo.create_child_of("子", "親")

        assert "親子関係作成エラー" in str(exc_info.value)

    def test_create_spouse_of_success(self, rel_repo, mock_client):
        """配偶者関係作成の成功テスト"""
        mock_client.execute.return_value = []

        rel_repo.create_spouse_of("配偶者1", "配偶者2")

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["person1_name"] == "配偶者1"
        assert params["person2_name"] == "配偶者2"
        assert params["is_current"] is True

    def test_create_spouse_of_with_dates(self, rel_repo, mock_client):
        """婚姻日・離婚日付きの配偶者関係作成テスト"""
        mock_client.execute.return_value = []
        marriage_date = date(2000, 4, 1)
        divorce_date = date(2010, 6, 15)

        rel_repo.create_spouse_of(
            "配偶者1",
            "配偶者2",
            marriage_date=marriage_date,
            divorce_date=divorce_date,
            is_current=False
        )

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["marriage_date"] == "2000-04-01"
        assert params["divorce_date"] == "2010-06-15"
        assert params["is_current"] is False

    def test_create_sibling_of_full_blood(self, rel_repo, mock_client):
        """全血兄弟姉妹関係作成テスト"""
        mock_client.execute.return_value = []

        rel_repo.create_sibling_of("兄", "妹", blood_type=BloodType.FULL)

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["person1_name"] == "兄"
        assert params["person2_name"] == "妹"
        assert params["blood_type"] == "full"
        assert params["shared_parent"] == "both"

    def test_create_sibling_of_half_blood(self, rel_repo, mock_client):
        """半血兄弟姉妹関係作成テスト"""
        mock_client.execute.return_value = []

        rel_repo.create_sibling_of("兄", "妹", blood_type=BloodType.HALF, shared_parent="mother")

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["blood_type"] == "half"
        assert params["shared_parent"] == "mother"

    def test_create_renounced_success(self, rel_repo, mock_client):
        """相続放棄関係作成の成功テスト"""
        mock_client.execute.return_value = []
        renounce_date = date(2025, 7, 1)

        rel_repo.create_renounced("放棄者", "被相続人", renounce_date, reason="経済的理由")

        call_args = mock_client.execute.call_args
        params = call_args[0][1]
        assert params["person_name"] == "放棄者"
        assert params["decedent_name"] == "被相続人"
        assert params["renounce_date"] == "2025-07-01"
        assert params["reason"] == "経済的理由"

    def test_find_children_success(self, rel_repo, mock_client):
        """子検索の成功テスト"""
        mock_children = [
            {"child": {"name": "子1", "is_alive": True, "is_decedent": False}},
            {"child": {"name": "子2", "is_alive": True, "is_decedent": False}}
        ]
        mock_client.execute.return_value = mock_children

        result = rel_repo.find_children("親")

        assert len(result) == 2
        assert result[0].name == "子1"
        assert result[1].name == "子2"

    def test_find_children_none(self, rel_repo, mock_client):
        """子が存在しない場合のテスト"""
        mock_client.execute.return_value = []

        result = rel_repo.find_children("親")

        assert len(result) == 0

    def test_find_parents_success(self, rel_repo, mock_client):
        """親検索の成功テスト"""
        mock_parents = [
            {"parent": {"name": "父", "is_alive": True, "is_decedent": False}},
            {"parent": {"name": "母", "is_alive": True, "is_decedent": False}}
        ]
        mock_client.execute.return_value = mock_parents

        result = rel_repo.find_parents("子")

        assert len(result) == 2
        assert result[0].name == "父"
        assert result[1].name == "母"

    def test_find_spouse_found(self, rel_repo, mock_client):
        """配偶者検索の成功テスト"""
        mock_spouse = [{"spouse": {"name": "配偶者", "is_alive": True, "is_decedent": False}}]
        mock_client.execute.return_value = mock_spouse

        result = rel_repo.find_spouse("本人")

        assert result is not None
        assert result.name == "配偶者"

    def test_find_spouse_not_found(self, rel_repo, mock_client):
        """配偶者が存在しない場合のテスト"""
        mock_client.execute.return_value = []

        result = rel_repo.find_spouse("本人")

        assert result is None

    def test_find_siblings_full_blood(self, rel_repo, mock_client):
        """全血兄弟姉妹検索のテスト"""
        mock_siblings = [
            {
                "sibling": {"name": "兄", "is_alive": True, "is_decedent": False},
                "r": {"blood_type": "full"}
            },
            {
                "sibling": {"name": "妹", "is_alive": True, "is_decedent": False},
                "r": {"blood_type": "full"}
            }
        ]
        mock_client.execute.return_value = mock_siblings

        result = rel_repo.find_siblings("本人")

        assert len(result) == 2
        assert result[0][0].name == "兄"
        assert result[0][1] == BloodType.FULL
        assert result[1][0].name == "妹"
        assert result[1][1] == BloodType.FULL

    def test_find_siblings_mixed_blood(self, rel_repo, mock_client):
        """全血・半血混在の兄弟姉妹検索のテスト"""
        mock_siblings = [
            {
                "sibling": {"name": "全血兄", "is_alive": True, "is_decedent": False},
                "r": {"blood_type": "full"}
            },
            {
                "sibling": {"name": "半血妹", "is_alive": True, "is_decedent": False},
                "r": {"blood_type": "half"}
            }
        ]
        mock_client.execute.return_value = mock_siblings

        result = rel_repo.find_siblings("本人")

        assert len(result) == 2
        assert result[0][1] == BloodType.FULL
        assert result[1][1] == BloodType.HALF


class TestInheritanceRepository:
    """InheritanceRepositoryのユニットテスト"""

    @pytest.fixture
    def mock_client(self):
        """モッククライアントのフィクスチャ"""
        return Mock()

    @pytest.fixture
    def inheritance_repo(self, mock_client):
        """InheritanceRepositoryのフィクスチャ"""
        return InheritanceRepository(mock_client)

    def test_get_spouse_found(self, inheritance_repo, mock_client):
        """配偶者取得の成功テスト"""
        mock_spouse = [{"spouse": {"name": "配偶者", "is_alive": True, "is_decedent": False}}]
        mock_client.execute.return_value = mock_spouse

        result = inheritance_repo.get_spouse()

        assert result is not None
        assert result.name == "配偶者"

    def test_get_spouse_not_found(self, inheritance_repo, mock_client):
        """配偶者が存在しない場合のテスト"""
        mock_client.execute.return_value = []

        result = inheritance_repo.get_spouse()

        assert result is None

    def test_get_first_rank_heirs_success(self, inheritance_repo, mock_client):
        """第1順位相続人取得の成功テスト"""
        mock_children = [
            {"child": {"name": "子1", "is_alive": True, "is_decedent": False}},
            {"child": {"name": "子2", "is_alive": True, "is_decedent": False}}
        ]
        mock_client.execute.return_value = mock_children

        result = inheritance_repo.get_first_rank_heirs()

        assert len(result) == 2
        assert result[0].name == "子1"
        assert result[1].name == "子2"

    def test_get_second_rank_heirs_success(self, inheritance_repo, mock_client):
        """第2順位相続人取得の成功テスト"""
        mock_ancestors = [
            {"ancestor": {"name": "父", "is_alive": True, "is_decedent": False}},
            {"ancestor": {"name": "母", "is_alive": True, "is_decedent": False}}
        ]
        mock_client.execute.return_value = mock_ancestors

        result = inheritance_repo.get_second_rank_heirs()

        assert len(result) == 2
        assert result[0].name == "父"
        assert result[1].name == "母"

    def test_get_third_rank_heirs_success(self, inheritance_repo, mock_client):
        """第3順位相続人取得の成功テスト"""
        mock_siblings = [
            {"sibling": {"name": "兄", "is_alive": True, "is_decedent": False}, "blood_type": "full"},
            {"sibling": {"name": "妹", "is_alive": True, "is_decedent": False}, "blood_type": "half"}
        ]
        mock_client.execute.return_value = mock_siblings

        result = inheritance_repo.get_third_rank_heirs()

        assert len(result) == 2
        assert result[0][0].name == "兄"
        assert result[0][1] == BloodType.FULL
        assert result[1][0].name == "妹"
        assert result[1][1] == BloodType.HALF

    def test_get_third_rank_heirs_empty(self, inheritance_repo, mock_client):
        """第3順位相続人が存在しない場合のテスト"""
        mock_client.execute.return_value = []

        result = inheritance_repo.get_third_rank_heirs()

        assert len(result) == 0

    def test_get_spouse_database_error(self, inheritance_repo, mock_client):
        """配偶者取得のデータベースエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Connection error")

        with pytest.raises(DatabaseException) as exc_info:
            inheritance_repo.get_spouse()

        assert "配偶者取得エラー" in str(exc_info.value)

    def test_get_first_rank_heirs_database_error(self, inheritance_repo, mock_client):
        """第1順位相続人取得のデータベースエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Query error")

        with pytest.raises(DatabaseException) as exc_info:
            inheritance_repo.get_first_rank_heirs()

        assert "第1順位相続人取得エラー" in str(exc_info.value)

    def test_get_second_rank_heirs_database_error(self, inheritance_repo, mock_client):
        """第2順位相続人取得のデータベースエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Query error")

        with pytest.raises(DatabaseException) as exc_info:
            inheritance_repo.get_second_rank_heirs()

        assert "第2順位相続人取得エラー" in str(exc_info.value)

    def test_get_third_rank_heirs_database_error(self, inheritance_repo, mock_client):
        """第3順位相続人取得のデータベースエラーテスト"""
        mock_client.execute.side_effect = Neo4jError("Query error")

        with pytest.raises(DatabaseException) as exc_info:
            inheritance_repo.get_third_rank_heirs()

        assert "第3順位相続人取得エラー" in str(exc_info.value)



"""Personモデルのテスト"""
from datetime import date
import pytest
from pydantic import ValidationError

from inheritance_calculator_core.models.person import Person, Gender


class TestPerson:
    """Personモデルの基本テスト"""

    def test_create_person_minimal(self):
        """最小限の情報でPersonを作成"""
        person = Person(name="山田太郎")

        assert person.name == "山田太郎"
        assert person.is_alive is True
        assert person.death_date is None
        assert person.is_decedent is False
        assert person.gender == Gender.UNKNOWN

    def test_create_person_full(self):
        """完全な情報でPersonを作成"""
        person = Person(
            name="山田花子",
            birth_date=date(1980, 5, 15),
            gender=Gender.FEMALE,
            is_alive=True
        )

        assert person.name == "山田花子"
        assert person.birth_date == date(1980, 5, 15)
        assert person.gender == Gender.FEMALE
        assert person.is_alive is True

    def test_name_required(self):
        """氏名が必須であることを検証"""
        with pytest.raises(ValidationError):
            Person()

    def test_name_min_length(self):
        """氏名の最小長の検証"""
        with pytest.raises(ValidationError):
            Person(name="")

    def test_birth_date_future_invalid(self):
        """未来の生年月日は不可"""
        from datetime import date as date_class, timedelta
        future_date = date_class.today() + timedelta(days=1)

        with pytest.raises(ValidationError) as exc_info:
            Person(name="未来人", birth_date=future_date)
        assert "Birth date cannot be in the future" in str(exc_info.value)


class TestPersonMethods:
    """Personモデルのメソッドテスト"""

    def test_mark_as_deceased(self):
        """死亡記録メソッドのテスト"""
        person = Person(name="山田太郎", is_alive=True)
        death_date = date(2025, 6, 15)

        person.mark_as_deceased(death_date)

        assert person.is_alive is False
        assert person.death_date == death_date
        assert person.updated_at is not None

    def test_mark_as_decedent(self):
        """被相続人記録メソッドのテスト"""
        person = Person(name="山田太郎")

        person.mark_as_decedent()

        assert person.is_decedent is True
        assert person.updated_at is not None


class TestPersonAge:
    """年齢計算のテスト"""

    def test_current_age_alive(self):
        """存命中の現在年齢計算"""
        # 45歳の人を想定（誕生日前）
        person = Person(
            name="山田太郎",
            birth_date=date(1980, 12, 31),  # 年末生まれ
            is_alive=True
        )

        age = person.current_age
        assert age is not None
        assert age >= 44  # 少なくとも44歳

    def test_current_age_no_birth_date(self):
        """生年月日不明の場合"""
        person = Person(name="山田太郎")

        assert person.current_age is None

    def test_age_at_death(self):
        """死亡時年齢の計算"""
        person = Person(
            name="山田太郎",
            birth_date=date(1950, 3, 10),
            is_alive=False,
            death_date=date(2025, 6, 15)
        )

        age = person.age_at_death
        assert age == 75

    def test_age_at_death_before_birthday(self):
        """誕生日前に死亡した場合の年齢計算"""
        person = Person(
            name="山田太郎",
            birth_date=date(1950, 10, 10),
            is_alive=False,
            death_date=date(2025, 6, 15)  # 誕生日前
        )

        age = person.age_at_death
        assert age == 74  # まだ75歳になっていない

    def test_age_at_death_missing_dates(self):
        """生年月日または死亡日が不明な場合"""
        person1 = Person(name="山田太郎", death_date=date(2025, 6, 15))
        assert person1.age_at_death is None

        person2 = Person(name="山田花子", birth_date=date(1950, 3, 10))
        assert person2.age_at_death is None


class TestPersonComparisons:
    """人物比較メソッドのテスト"""

    def test_is_older_than(self):
        """年上判定のテスト"""
        older = Person(name="兄", birth_date=date(1980, 1, 1))
        younger = Person(name="弟", birth_date=date(1985, 1, 1))

        assert older.is_older_than(younger) is True
        assert younger.is_older_than(older) is False

    def test_is_older_than_no_birth_date(self):
        """生年月日不明の場合の比較"""
        person1 = Person(name="山田太郎")
        person2 = Person(name="山田花子", birth_date=date(1980, 1, 1))

        assert person1.is_older_than(person2) is False
        assert person2.is_older_than(person1) is False

    def test_died_before(self):
        """死亡順序の判定"""
        person1 = Person(
            name="先に死亡",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        person2 = Person(
            name="後に死亡",
            is_alive=False,
            death_date=date(2025, 1, 1)
        )

        assert person1.died_before(person2) is True
        assert person2.died_before(person1) is False

    def test_died_before_one_alive(self):
        """片方が存命の場合"""
        deceased = Person(
            name="故人",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        alive = Person(name="存命", is_alive=True)

        assert deceased.died_before(alive) is True
        assert alive.died_before(deceased) is False


class TestPersonStringRepresentation:
    """文字列表現のテスト"""

    def test_str_alive(self):
        """存命の場合の文字列表現"""
        person = Person(name="山田太郎", is_alive=True)
        result = str(person)

        assert "山田太郎" in result
        assert "存命" in result

    def test_str_deceased(self):
        """故人の場合の文字列表現"""
        person = Person(name="山田花子", is_alive=False)
        result = str(person)

        assert "山田花子" in result
        assert "故人" in result

    def test_str_decedent(self):
        """被相続人の場合の文字列表現"""
        person = Person(name="山田太郎", is_decedent=True)
        result = str(person)

        assert "山田太郎" in result
        assert "被相続人" in result

    def test_repr(self):
        """デバッグ用文字列表現"""
        person = Person(name="山田太郎", is_alive=True, is_decedent=False)
        result = repr(person)

        assert "Person" in result
        assert "name='山田太郎'" in result
        assert "is_alive=True" in result
        assert "is_decedent=False" in result

    def test_str_with_age_alive(self):
        """存命中で年齢ありの文字列表現"""
        person = Person(
            name="山田太郎",
            birth_date=date(1980, 5, 15),
            is_alive=True
        )
        result = str(person)

        assert "山田太郎" in result
        assert "存命" in result
        assert "歳" in result
        # 年齢は計算されるので具体的な値は検証しない

    def test_str_with_age_deceased(self):
        """故人で享年ありの文字列表現"""
        person = Person(
            name="山田花子",
            birth_date=date(1950, 3, 10),
            death_date=date(2025, 6, 15),
            is_alive=False
        )
        result = str(person)

        assert "山田花子" in result
        assert "故人" in result
        assert "享年75歳" in result

    def test_str_without_birth_date(self):
        """生年月日なしの文字列表現（年齢表示なし）"""
        person = Person(name="山田次郎", is_alive=True)
        result = str(person)

        assert "山田次郎" in result
        assert "存命" in result
        assert "歳" not in result  # 年齢情報がないので「歳」は含まれない



"""Person モデルの連絡先情報テスト

連絡先フィールド（address, phone, email）のテスト
"""
import pytest
from pydantic import ValidationError

from inheritance_calculator_core.models.person import Person, Gender


class TestPersonContactInfo:
    """Person モデルの連絡先情報テスト"""

    def test_create_person_with_contact_info(self):
        """連絡先情報を含むPerson作成テスト"""
        person = Person(
            name="山田太郎",
            address="東京都渋谷区渋谷1-1-1",
            phone="03-1234-5678",
            email="taro@example.com"
        )

        assert person.name == "山田太郎"
        assert person.address == "東京都渋谷区渋谷1-1-1"
        assert person.phone == "03-1234-5678"
        assert person.email == "taro@example.com"

    def test_create_person_without_contact_info(self):
        """連絡先情報なしでPerson作成テスト（後方互換性）"""
        person = Person(name="山田花子")

        assert person.name == "山田花子"
        assert person.address is None
        assert person.phone is None
        assert person.email is None

    def test_create_person_with_partial_contact_info(self):
        """部分的な連絡先情報でPerson作成テスト"""
        person = Person(
            name="山田一郎",
            address="大阪府大阪市北区梅田1-1-1",
            # phone and email are None
        )

        assert person.address == "大阪府大阪市北区梅田1-1-1"
        assert person.phone is None
        assert person.email is None

    def test_valid_email_format(self):
        """有効なメールアドレス形式のテスト"""
        valid_emails = [
            "test@example.com",
            "user.name@example.co.jp",
            "user+tag@subdomain.example.com",
        ]

        for email in valid_emails:
            person = Person(name="テストユーザー", email=email)
            assert person.email == email

    def test_invalid_email_format(self):
        """無効なメールアドレス形式のテスト"""
        invalid_emails = [
            "not-an-email",
            "@example.com",
            "user@",
            "user @example.com",  # space
        ]

        for email in invalid_emails:
            with pytest.raises(ValidationError):
                Person(name="テストユーザー", email=email)

    def test_phone_number_flexibility(self):
        """電話番号の柔軟な形式テスト"""
        valid_phones = [
            "03-1234-5678",
            "0312345678",
            "090-1234-5678",
            "09012345678",
            "+81-3-1234-5678",
            "+81312345678",
        ]

        for phone in valid_phones:
            person = Person(name="テストユーザー", phone=phone)
            assert person.phone == phone

    def test_address_flexibility(self):
        """住所の柔軟性テスト"""
        addresses = [
            "東京都渋谷区渋谷1-1-1",
            "〒150-0002 東京都渋谷区渋谷1-1-1 マンション名101号室",
            "大阪府大阪市北区梅田1-1-1\n(注: 郵便物はこちらへ)",  # 複数行
        ]

        for address in addresses:
            person = Person(name="テストユーザー", address=address)
            assert person.address == address

    def test_set_contact_info_method(self):
        """set_contact_info メソッドのテスト"""
        person = Person(name="山田太郎")

        # 初期状態は None
        assert person.address is None
        assert person.phone is None
        assert person.email is None

        # 連絡先情報を設定
        person.set_contact_info(
            address="東京都渋谷区渋谷1-1-1",
            phone="03-1234-5678",
            email="taro@example.com"
        )

        assert person.address == "東京都渋谷区渋谷1-1-1"
        assert person.phone == "03-1234-5678"
        assert person.email == "taro@example.com"

    def test_set_contact_info_partial(self):
        """set_contact_info で部分的に設定するテスト"""
        person = Person(
            name="山田花子",
            address="初期住所",
            phone="初期電話",
            email="initial@example.com"
        )

        # 住所のみ更新
        person.set_contact_info(address="新しい住所")

        assert person.address == "新しい住所"
        assert person.phone == "初期電話"  # 変更されない
        assert person.email == "initial@example.com"  # 変更されない

    def test_contact_info_serialization(self):
        """連絡先情報のシリアライゼーションテスト"""
        person = Person(
            name="山田太郎",
            address="東京都渋谷区渋谷1-1-1",
            phone="03-1234-5678",
            email="taro@example.com"
        )

        # model_dump()で辞書化
        data = person.model_dump()

        assert data["name"] == "山田太郎"
        assert data["address"] == "東京都渋谷区渋谷1-1-1"
        assert data["phone"] == "03-1234-5678"
        assert data["email"] == "taro@example.com"

    def test_contact_info_deserialization(self):
        """連絡先情報のデシリアライゼーションテスト"""
        data = {
            "name": "山田花子",
            "address": "大阪府大阪市北区梅田1-1-1",
            "phone": "06-1234-5678",
            "email": "hanako@example.com"
        }

        person = Person(**data)

        assert person.name == "山田花子"
        assert person.address == "大阪府大阪市北区梅田1-1-1"
        assert person.phone == "06-1234-5678"
        assert person.email == "hanako@example.com"

    def test_backward_compatibility_deserialization(self):
        """後方互換性: 連絡先フィールドなしのデータからデシリアライゼーション"""
        # 既存データ（連絡先フィールドなし）
        legacy_data = {
            "name": "山田一郎",
            "is_alive": True,
            "gender": "male"
        }

        # エラーなく作成できる
        person = Person(**legacy_data)

        assert person.name == "山田一郎"
        assert person.is_alive is True
        assert person.gender == Gender.MALE
        assert person.address is None
        assert person.phone is None
        assert person.email is None



"""サービスファクトリ

サービスインスタンスを生成し、依存関係を自動的に注入する。
"""
from typing import Optional

from .heir_validator import HeirValidator
from .share_calculator import ShareCalculator
from .inheritance_calculator import InheritanceCalculator


class ServiceFactory:
    """
    サービスファクトリ

    サービスインスタンスの生成と依存性注入を管理する。
    シングルトンパターンでサービスインスタンスを再利用可能。
    """

    def __init__(self) -> None:
        """初期化"""
        self._heir_validator: Optional[HeirValidator] = None
        self._share_calculator: Optional[ShareCalculator] = None
        self._inheritance_calculator: Optional[InheritanceCalculator] = None

    def get_heir_validator(self) -> HeirValidator:
        """
        HeirValidatorインスタンスを取得

        Returns:
            HeirValidatorインスタンス（シングルトン）
        """
        if self._heir_validator is None:
            self._heir_validator = HeirValidator()
        return self._heir_validator

    def get_share_calculator(self) -> ShareCalculator:
        """
        ShareCalculatorインスタンスを取得

        Returns:
            ShareCalculatorインスタンス（シングルトン）
        """
        if self._share_calculator is None:
            self._share_calculator = ShareCalculator()
        return self._share_calculator

    def get_inheritance_calculator(self) -> InheritanceCalculator:
        """
        InheritanceCalculatorインスタンスを取得

        依存するHeirValidatorとShareCalculatorを自動的に注入する。

        Returns:
            InheritanceCalculatorインスタンス（シングルトン）
        """
        if self._inheritance_calculator is None:
            self._inheritance_calculator = InheritanceCalculator(
                validator=self.get_heir_validator(),
                calculator=self.get_share_calculator()
            )
        return self._inheritance_calculator

    def reset(self) -> None:
        """
        全てのキャッシュされたインスタンスをリセット

        主にテスト用途で使用。
        """
        self._heir_validator = None
        self._share_calculator = None
        self._inheritance_calculator = None



"""値オブジェクト

ドメインモデルで使用する値オブジェクトの定義。
"""
from dataclasses import dataclass
from uuid import UUID, uuid4
from typing import Union


@dataclass(frozen=True)
class PersonID:
    """
    人物の一意識別子（値オブジェクト）

    不変な識別子として、辞書のキーやハッシュ可能なコンテナで使用できる。
    """

    value: UUID

    @classmethod
    def generate(cls) -> 'PersonID':
        """
        新しいPersonIDを生成

        Returns:
            新しいPersonIDインスタンス
        """
        return cls(value=uuid4())

    @classmethod
    def from_string(cls, id_str: str) -> 'PersonID':
        """
        文字列からPersonIDを生成

        Args:
            id_str: UUID文字列

        Returns:
            PersonIDインスタンス

        Raises:
            ValueError: 無効なUUID文字列の場合
        """
        return cls(value=UUID(id_str))

    @classmethod
    def from_uuid(cls, uuid_val: UUID) -> 'PersonID':
        """
        UUIDからPersonIDを生成

        Args:
            uuid_val: UUIDオブジェクト

        Returns:
            PersonIDインスタンス
        """
        return cls(value=uuid_val)

    def __str__(self) -> str:
        """
        文字列表現

        Returns:
            UUID文字列
        """
        return str(self.value)

    def __hash__(self) -> int:
        """
        ハッシュ値

        Returns:
            ハッシュ値（辞書のキーとして使用可能）
        """
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        """
        等値比較

        Args:
            other: 比較対象

        Returns:
            PersonIDが等しい場合True
        """
        if not isinstance(other, PersonID):
            return False
        return self.value == other.value

    def __repr__(self) -> str:
        """
        デバッグ用文字列表現

        Returns:
            デバッグ用文字列
        """
        return f"PersonID('{self.value}')"



"""サービス層の基底クラス"""
from abc import ABC
from typing import Any, Generic, TypeVar

from inheritance_calculator_core.utils.logger import get_logger


T = TypeVar('T')


class BaseService(ABC, Generic[T]):
    """サービスの基底クラス"""

    def __init__(self) -> None:
        """サービスを初期化"""
        self.logger = get_logger(self.__class__.__name__)

    def log_operation(self, operation: str, **kwargs: Any) -> None:
        """
        操作をログに記録

        Args:
            operation: 操作名
            **kwargs: ログに含める追加情報
        """
        self.logger.info(f"{operation}: {kwargs}")

    def log_error(self, error: Exception, context: str = "") -> None:
        """
        エラーをログに記録

        Args:
            error: 発生した例外
            context: エラーのコンテキスト情報
        """
        error_msg = f"Error in {context}: {error}" if context else f"Error: {error}"
        self.logger.error(error_msg, exc_info=True)

    def log_info(self, message: str, **kwargs: Any) -> None:
        """
        情報をログに記録

        Args:
            message: 情報メッセージ
            **kwargs: ログに含める追加情報
        """
        self.logger.info(f"{message}: {kwargs}" if kwargs else message)

    def log_warning(self, message: str, **kwargs: Any) -> None:
        """
        警告をログに記録

        Args:
            message: 警告メッセージ
            **kwargs: ログに含める追加情報
        """
        self.logger.warning(f"{message}: {kwargs}" if kwargs else message)

    def log_debug(self, message: str, **kwargs: Any) -> None:
        """
        デバッグ情報をログに記録

        Args:
            message: デバッグメッセージ
            **kwargs: ログに含める追加情報
        """
        self.logger.debug(f"{message}: {kwargs}" if kwargs else message)



"""ベースモデルクラス"""
from datetime import datetime
from typing import Any, Dict, Optional, Union
from uuid import UUID, uuid4

from pydantic import BaseModel, Field, ConfigDict, field_validator

from .value_objects import PersonID


class BaseEntity(BaseModel):
    """全エンティティの基底クラス"""

    model_config = ConfigDict(
        # JSON互換性
        json_encoders={datetime: lambda v: v.isoformat()},
        # バリデーション設定
        validate_assignment=True,
        # 追加フィールドを許可しない
        extra='forbid',
    )

    id: PersonID = Field(default_factory=lambda: PersonID.generate(), description="エンティティID")
    created_at: datetime = Field(default_factory=datetime.now, description="作成日時")
    updated_at: Optional[datetime] = Field(default=None, description="更新日時")

    @field_validator('id', mode='before')
    @classmethod
    def convert_uuid_to_person_id(cls, v: Union[UUID, PersonID, str]) -> PersonID:
        """
        UUIDをPersonIDに変換（後方互換性のため）

        Args:
            v: UUID、PersonID、または文字列

        Returns:
            PersonID
        """
        if isinstance(v, PersonID):
            return v
        elif isinstance(v, UUID):
            return PersonID.from_uuid(v)
        elif isinstance(v, str):
            return PersonID.from_string(v)
        else:
            raise ValueError(f"Invalid type for id: {type(v)}")

    def mark_updated(self) -> None:
        """更新日時を記録"""
        self.updated_at = datetime.now()


class Neo4jNode(BaseEntity):
    """Neo4jノードの基底クラス"""

    @property
    def neo4j_labels(self) -> list[str]:
        """Neo4jラベルのリスト"""
        return [self.__class__.__name__]

    @property
    def neo4j_properties(self) -> Dict[str, Any]:
        """Neo4jプロパティの辞書"""
        return self.model_dump(exclude={'id'})


class Neo4jRelationship(BaseModel):
    """Neo4jリレーションシップの基底クラス"""

    model_config = ConfigDict(
        validate_assignment=True,
        extra='forbid',
    )

    @property
    def neo4j_type(self) -> str:
        """Neo4jリレーションシップタイプ"""
        return self.__class__.__name__.upper()

    @property
    def neo4j_properties(self) -> Dict[str, Any]:
        """Neo4jプロパティの辞書"""
        return self.model_dump()



"""値オブジェクトのテスト"""
from uuid import UUID, uuid4
import pytest

from inheritance_calculator_core.models.value_objects import PersonID


class TestPersonID:
    """PersonID値オブジェクトのテスト"""

    def test_generate(self) -> None:
        """新しいPersonIDを生成できる"""
        person_id = PersonID.generate()
        assert isinstance(person_id, PersonID)
        assert isinstance(person_id.value, UUID)

    def test_generate_creates_unique_ids(self) -> None:
        """generate()は毎回異なるIDを生成する"""
        id1 = PersonID.generate()
        id2 = PersonID.generate()
        assert id1 != id2

    def test_from_string(self) -> None:
        """文字列からPersonIDを生成できる"""
        uuid_str = "550e8400-e29b-41d4-a716-446655440000"
        person_id = PersonID.from_string(uuid_str)
        assert isinstance(person_id, PersonID)
        assert person_id.value == UUID(uuid_str)

    def test_from_string_invalid_raises_error(self) -> None:
        """無効な文字列からPersonIDを生成しようとするとエラー"""
        with pytest.raises(ValueError):
            PersonID.from_string("invalid-uuid")

    def test_from_uuid(self) -> None:
        """UUIDからPersonIDを生成できる"""
        uuid_val = uuid4()
        person_id = PersonID.from_uuid(uuid_val)
        assert isinstance(person_id, PersonID)
        assert person_id.value == uuid_val

    def test_str(self) -> None:
        """文字列表現を取得できる"""
        uuid_str = "550e8400-e29b-41d4-a716-446655440000"
        person_id = PersonID.from_string(uuid_str)
        assert str(person_id) == uuid_str

    def test_repr(self) -> None:
        """デバッグ用文字列表現を取得できる"""
        uuid_str = "550e8400-e29b-41d4-a716-446655440000"
        person_id = PersonID.from_string(uuid_str)
        assert repr(person_id) == f"PersonID('{uuid_str}')"

    def test_equality(self) -> None:
        """等値比較ができる"""
        uuid_str = "550e8400-e29b-41d4-a716-446655440000"
        id1 = PersonID.from_string(uuid_str)
        id2 = PersonID.from_string(uuid_str)
        id3 = PersonID.generate()

        assert id1 == id2
        assert id1 != id3
        assert id2 != id3

    def test_equality_with_non_person_id(self) -> None:
        """PersonID以外との比較でFalse"""
        person_id = PersonID.generate()
        assert person_id != "not a PersonID"
        assert person_id != 123
        assert person_id != None

    def test_hashable(self) -> None:
        """ハッシュ可能で辞書のキーとして使える"""
        id1 = PersonID.generate()
        id2 = PersonID.generate()

        # 辞書のキーとして使用
        data = {id1: "value1", id2: "value2"}
        assert data[id1] == "value1"
        assert data[id2] == "value2"

    def test_hashable_same_ids_have_same_hash(self) -> None:
        """同じUUIDを持つPersonIDは同じハッシュ値を持つ"""
        uuid_str = "550e8400-e29b-41d4-a716-446655440000"
        id1 = PersonID.from_string(uuid_str)
        id2 = PersonID.from_string(uuid_str)

        assert hash(id1) == hash(id2)

    def test_immutable(self) -> None:
        """値オブジェクトは不変"""
        person_id = PersonID.generate()

        # frozen=Trueなので属性変更は不可
        with pytest.raises(AttributeError):
            person_id.value = uuid4()  # type: ignore

    def test_can_be_used_in_set(self) -> None:
        """セットの要素として使用できる"""
        id1 = PersonID.generate()
        id2 = PersonID.generate()
        id3 = PersonID.from_string(str(id1))  # id1と同じ

        id_set = {id1, id2, id3}
        # id1とid3は同じなので、セットには2要素のみ
        assert len(id_set) == 2
        assert id1 in id_set
        assert id2 in id_set
        assert id3 in id_set



"""データベース層の基底クラスとインターフェース"""
from abc import ABC, abstractmethod
from typing import Generic, TypeVar, Optional, List, Any, Dict

from ..models.value_objects import PersonID


T = TypeVar('T')


class Repository(ABC, Generic[T]):
    """リポジトリパターンの基底クラス"""

    @abstractmethod
    async def create(self, entity: T) -> T:
        """
        エンティティを作成

        Args:
            entity: 作成するエンティティ

        Returns:
            作成されたエンティティ

        Raises:
            DatabaseConnectionError: データベース接続エラー
            ValidationError: バリデーションエラー
        """
        pass

    @abstractmethod
    async def get_by_id(self, entity_id: PersonID) -> Optional[T]:
        """
        IDでエンティティを取得

        Args:
            entity_id: エンティティID

        Returns:
            エンティティ（存在しない場合はNone）

        Raises:
            DatabaseConnectionError: データベース接続エラー
        """
        pass

    @abstractmethod
    async def update(self, entity: T) -> T:
        """
        エンティティを更新

        Args:
            entity: 更新するエンティティ

        Returns:
            更新されたエンティティ

        Raises:
            DatabaseConnectionError: データベース接続エラー
            ValidationError: バリデーションエラー
        """
        pass

    @abstractmethod
    async def delete(self, entity_id: PersonID) -> bool:
        """
        エンティティを削除

        Args:
            entity_id: エンティティID

        Returns:
            削除成功の場合True

        Raises:
            DatabaseConnectionError: データベース接続エラー
        """
        pass

    @abstractmethod
    async def list_all(self, limit: Optional[int] = None, offset: int = 0) -> List[T]:
        """
        全エンティティを取得

        Args:
            limit: 取得件数制限
            offset: 取得開始位置

        Returns:
            エンティティのリスト

        Raises:
            DatabaseConnectionError: データベース接続エラー
        """
        pass


class BaseRepository(ABC, Generic[T]):
    """
    同期版リポジトリの基底クラス

    Note:
        現在の実装はNeo4jの同期ドライバーを使用しているため、
        同期メソッドとして定義しています。
    """

    @abstractmethod
    def create(self, entity: T) -> T:
        """
        エンティティを作成

        Args:
            entity: 作成するエンティティ

        Returns:
            作成されたエンティティ

        Raises:
            DatabaseException: データベースエラー
        """
        pass

    @abstractmethod
    def find_by_id(self, entity_id: PersonID) -> Optional[T]:
        """
        IDでエンティティを取得

        Args:
            entity_id: エンティティID

        Returns:
            エンティティ（存在しない場合はNone）

        Raises:
            DatabaseException: データベースエラー
        """
        pass

    @abstractmethod
    def update(self, entity: T) -> T:
        """
        エンティティを更新

        Args:
            entity: 更新するエンティティ

        Returns:
            更新されたエンティティ

        Raises:
            DatabaseException: データベースエラー
        """
        pass

    @abstractmethod
    def delete(self, entity_id: PersonID) -> bool:
        """
        エンティティを削除

        Args:
            entity_id: エンティティID

        Returns:
            削除成功の場合True

        Raises:
            DatabaseException: データベースエラー
        """
        pass

    @abstractmethod
    def find_all(self, limit: Optional[int] = None, offset: int = 0) -> List[T]:
        """
        全エンティティを取得

        Args:
            limit: 取得件数制限
            offset: 取得開始位置

        Returns:
            エンティティのリスト

        Raises:
            DatabaseException: データベースエラー
        """
        pass


class DatabaseClient(ABC):
    """データベースクライアントの基底クラス（同期版）

    Note:
        現在の実装ではNeo4jの同期ドライバーを使用しているため、
        すべてのメソッドは同期版として定義されています。
        将来的に非同期対応が必要になった場合は、
        AsyncDatabaseClientクラスを別途定義することを推奨します。
    """

    @abstractmethod
    def connect(self) -> None:
        """
        データベースに接続

        Raises:
            DatabaseConnectionError: 接続エラー
        """
        pass

    @abstractmethod
    def disconnect(self) -> None:
        """
        データベース接続を切断

        Raises:
            DatabaseConnectionError: 切断エラー
        """
        pass

    @abstractmethod
    def execute_query(
        self,
        query: str,
        parameters: Optional[dict[str, Any]] = None
    ) -> Any:
        """
        クエリを実行

        Args:
            query: 実行するクエリ
            parameters: クエリパラメータ

        Returns:
            クエリ実行結果

        Raises:
            DatabaseConnectionError: 実行エラー
        """
        pass

    @abstractmethod
    def health_check(self) -> bool:
        """
        ヘルスチェック

        Returns:
            正常な場合True

        Raises:
            DatabaseConnectionError: 接続エラー
        """
        pass

    @abstractmethod
    def begin_transaction(self) -> Any:
        """
        トランザクションを開始

        Returns:
            トランザクションオブジェクト

        Raises:
            DatabaseConnectionError: トランザクション開始エラー
        """
        pass

    @abstractmethod
    def commit_transaction(self, transaction: Any) -> None:
        """
        トランザクションをコミット

        Args:
            transaction: トランザクションオブジェクト

        Raises:
            DatabaseConnectionError: コミットエラー
        """
        pass

    @abstractmethod
    def rollback_transaction(self, transaction: Any) -> None:
        """
        トランザクションをロールバック

        Args:
            transaction: トランザクションオブジェクト

        Raises:
            DatabaseConnectionError: ロールバックエラー
        """
        pass

    def __enter__(self) -> "DatabaseClient":
        """コンテキストマネージャーの開始"""
        self.connect()
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """コンテキストマネージャーの終了"""
        self.disconnect()
